@@ -18,13 +18,9 @@ header file containing all the main procedures to interface with the ICQ server
 #include "licq_user.h"
 #include "licq_plugind.h"
 #include "licq_color.h"
-
-#ifdef PROTOCOL_PLUGIN
 #include "licq_protoplugind.h"
 
 class CProtoPlugin;
-#endif
-
 class CPlugin;
 class CPacket;
 class CPacketTcp;
@@ -36,6 +32,8 @@ class TCPSocket;
 class SrvSocket;
 class INetSocket;
 class ProxyServer;
+class CReverseConnectToUserData;
+class CMSN;
 
 const unsigned short IGNORE_MASSMSG    = 1;
 const unsigned short IGNORE_NEWUSERS   = 2;
@@ -43,12 +41,16 @@ const unsigned short IGNORE_EMAILPAGER = 4;
 const unsigned short IGNORE_WEBPANEL   = 8;
 
 //-----Stats-----------------------------------------------------------------
+//! Keeps information about statistics of the daemon
 class CDaemonStats
 {
 public:
   // Accessors
+  //! Total number of events.
   unsigned long Total() { return m_nTotal; }
+  //! Total number of events for the current day only.
   unsigned long Today() { return m_nTotal - m_nOriginal; }
+  //! Name of the kind of statistic.
   const char *Name()    { return m_szName; }
 
 protected:
@@ -70,6 +72,7 @@ protected:
   char m_szName[32];
 
 friend class CICQDaemon;
+friend class CMSN;
 };
 
 typedef std::vector<CDaemonStats> DaemonStatsList;
@@ -96,22 +99,85 @@ public:
   pthread_t *Shutdown();
   void SaveConf();
 
-#ifdef PROTOCOL_PLUGIN
   // GUI Plugins call these now
+  //! Add a user to the local contact list.
+  /*!
+      \param szId The user ID to add.
+      \param nPPID The user's protocol plugin ID.
+      \param _bAuthRequired True if we need to get authorization first.
+  */
   void ProtoAddUser(const char *szId, unsigned long nPPID, bool _bAuthRequired = false);
 
+  //! Remove a user from the local contact list.
+  /*!
+      \param szId The user ID to remove.
+      \param nPPID The user's protocol plugin ID.
+  */
   void ProtoRemoveUser(const char *szId, unsigned long nPPID);
 
+  //! Rename a user on the server contact list.
+  /*!
+       \param szId The user ID to rename.
+       \param nPPID The user's protocol plugin ID.
+  */
+  void ProtoRenameUser(const char *szId, unsigned long nPPID);
+
+  //! Change status for a protocol.
+  /*!
+       \param nPPID The protocol ID.
+       \param nNewStatus The status to change to.
+  */
   unsigned long ProtoSetStatus(unsigned long nPPID, unsigned short nNewStatus);
 
+  //! Logon for a protocol.
+  /*!
+        \param nPPID The protocol ID.
+        \param nLogonStatus The initial status to set after logon is complete.
+  */
   unsigned long ProtoLogon(unsigned long nPPID, unsigned short nLogonStatus);
-  
+
+  //! Logoff from a protocol.
+  /*!
+        \param nPPID The protocol ID.
+  */
   void ProtoLogoff(unsigned long nPPID);
 
+  //! Send typing notification.
+
+  /*!
+        \param szId The user ID.
+        \param nPPID The protocol ID.
+        \param bActive The state of the typing notification. TRUE if active.
+  */
+  void ProtoTypingNotification(const char *szId, unsigned long nPPID, bool Active);
+
+  //! Send a message to a user on this protocol.
+  /*!
+        \param szId The user ID.
+        \param nPPID The protocol ID.
+        \param szMessage The message to be sent.
+        \param bOnline True if the user is online.
+        \param nLevel Any special flags (protocol specific).
+        \param bMultipleRecipients True if sending the same message to
+          more than one user.
+        \param pColor The color of the text and background.
+  */
   unsigned long ProtoSendMessage(const char *szId, unsigned long nPPID,
      const char *szMessage, bool bOnline, unsigned short nLevel,
      bool bMultipleRecipients = false, CICQColor *pColor = NULL);
 
+  //! Send a URL to a user on this protocol
+  /*!
+      This may not be available for all protocols.
+        \param szId The user ID.
+        \param szUrl The URL to be sent.
+        \param szDescription A description of the URL that can be sent.
+        \param bOnline True if the user is online.
+        \param nLevel Any special flags (protocol specific).
+        \param bMultipleRecipients True if sending the same URL to
+          more than one user.
+        \param pColor The color of the text and background.
+  */      
   unsigned long ProtoSendUrl(const char *szId, unsigned long nPPID,
      const char *szUrl, const char *szDescription, bool bOnline,
      unsigned short nLevel, bool bMultipleRecipients = false,
@@ -136,10 +202,19 @@ public:
      unsigned long nMsgID[], bool bDirect);
   void ProtoChatRequestCancel(const char *szId, unsigned long nPPID,
      unsigned long nSequence);
-#endif
+  
+  unsigned long ProtoAuthorizeGrant(const char *szId, unsigned long nPPID,
+     const char *szMessage);
+
+  unsigned long ProtoAuthorizeRefuse(const char *szId, unsigned long nPPID,
+     const char *szMessage);
 
   // TCP (user) functions
   // Message
+  unsigned long icqSendMessage(const char *szId, const char *szMessage,
+     bool bOnline, unsigned short nLevel, bool bMultipleRecipients = false,
+     CICQColor *pColor = NULL);
+
   unsigned long icqSendMessage(unsigned long nUin, const char *szMessage,
      bool bOnline, unsigned short nLevel, bool bMultipleRecipients = false,
      CICQColor *pColor = NULL);
@@ -148,11 +223,16 @@ public:
      const char *szDescription, bool bOnline, unsigned short nLevel,
      bool bMultipleRecipients = false, CICQColor *pColor = NULL);
   // Contact List
+  unsigned long icqSendContactList(const char *szId, UserStringList &users,
+     bool bOnline, unsigned short nLevel, bool bMultipleRecipients = false,
+     CICQColor *pColor = NULL);
+
   unsigned long icqSendContactList(unsigned long nUin, UinList &uins,
      bool bOnline, unsigned short nLevel, bool bMultipleRecipients = false,
      CICQColor *pColor = NULL);
   // Auto Response
-  unsigned long icqFetchAutoResponse(unsigned long nUin, bool bServer = false);
+  unsigned long icqFetchAutoResponse(unsigned long nUin, bool bServer = false) __attribute__ ((deprecated));
+  unsigned long icqFetchAutoResponse(const char *_szId, unsigned long _nPPID, bool bServer = false);
   // Chat Request
   unsigned long icqChatRequest(unsigned long nUin, const char *szReason,
      unsigned short nLevel, bool bServer);
@@ -177,12 +257,27 @@ public:
   unsigned long icqCloseSecureChannel(unsigned long nUin);
   void icqOpenSecureChannelCancel(unsigned long nUin, unsigned long nSequence);
 
+  // Plugins
+  unsigned long icqRequestInfoPluginList(const char *szId,
+     bool bServer = false);
+  unsigned long icqRequestPhoneBook(const char *szId, bool bServer = false);
+  unsigned long icqRequestPicture(const char *szId, bool bServer = false);
+  unsigned long icqRequestStatusPluginList(const char *szId,
+     bool bServer = false);
+  unsigned long icqRequestSharedFiles(const char *szId, bool bServer = false);
+  unsigned long icqRequestPhoneFollowMe(const char *szId,
+     bool bServer = false);
+  unsigned long icqRequestICQphone(const char *szId, bool bServer = false);
+
   // Server functions
   void icqRegister(const char *_szPasswd);
   unsigned long icqFetchAutoResponseServer(unsigned long);
   unsigned long icqLogon(unsigned short logonStatus);
+  unsigned long icqUserBasicInfo(const char *);
   unsigned long icqUserBasicInfo(unsigned long);
+  unsigned long icqUserExtendedInfo(const char *);
   unsigned long icqUserExtendedInfo(unsigned long);
+  unsigned long icqRequestMetaInfo(const char *);
   unsigned long icqRequestMetaInfo(unsigned long);
 
   unsigned long icqUpdateBasicInfo(const char *, const char *, const char *,
@@ -195,7 +290,8 @@ public:
                            const char *_szFax, const char *_szAddress,
 			   const char *_szZip, unsigned short _nCompanyCountry,
                            const char *_szName, const char *_szDepartment,
-                           const char *_szPosition, const char *_szHomepage);
+                           const char *_szPosition, unsigned short _nCompanyOccupation,
+                           const char *_szHomepage);
   unsigned long icqSetGeneralInfo(const char *szAlias, const char *szFirstName,
                               const char *szLastName, const char *szEmailPrimary,
                               const char *szCity,
@@ -210,6 +306,11 @@ public:
                            char nBirthDay, char nLanguage1,
                            char nLanguage2, char nLanguage3);
   unsigned long icqSetSecurityInfo(bool bAuthorize, bool bHideIp, bool bWebAware);
+  unsigned long icqSetInterestsInfo(const ICQUserCategory *interests);
+  unsigned long icqSetOrgBackInfo(const ICQUserCategory *orgs,
+                                  const ICQUserCategory *background);
+  unsigned long icqSetHomepageInfo(bool bCatetory, unsigned short nCategory,
+                                const char *szHomepageDesc, bool bICQHomepage);
   unsigned long icqSetAbout(const char *szAbout);
   unsigned long icqSetPassword(const char *szPassword);
   unsigned long icqSetStatus(unsigned short newStatus);
@@ -230,54 +331,78 @@ public:
   void postLogoff(int nSD, ICQEvent *cancelledEvent);
   void icqRelogon();
   unsigned long icqAuthorizeGrant(unsigned long nUin, const char *szMessage);
+  unsigned long icqAuthorizeGrant(const char *szId, const char *szMessage);
   unsigned long icqAuthorizeRefuse(unsigned long nUin, const char *szMessage);
+  unsigned long icqAuthorizeRefuse(const char *szId, const char *szMessage);
   void icqRequestAuth(unsigned long _nUin, const char *_szMessage);
   void icqAlertUser(unsigned long _nUin);
-  void icqAddUser(unsigned long, bool _bAuthReq = false);
-  void icqAddUserServer(unsigned long, bool);
+  void icqAddUser(unsigned long _nUin, bool _bAuthReq = false);
+  void icqAddUser(const char *_szId, bool _bAuthReq = false);
+  void icqAddUserServer(const char *_szId, bool _bAuthReq);
+  void icqAddUserServer(unsigned long _nUin, bool _bAuthReq);
   void icqAddGroup(const char *);
   void icqRemoveUser(unsigned long _nUin);
+  void icqRemoveUser(const char *);
   void icqRemoveGroup(const char *);
   void icqChangeGroup(unsigned long _nUin, unsigned short _nNewGroup,
                       unsigned short _nOldGSID, unsigned short _nNewType,
                       unsigned short _nOldType);
+  void icqChangeGroup(const char *_szId, unsigned long _nPPID,
+                      unsigned short _nNewGroup, unsigned short _nOldGSID,
+                      unsigned short _nNewType, unsigned short _nOldType);
   void icqRenameGroup(const char *_szNewName, unsigned short _nGSID);
   void icqRenameUser(unsigned long _nUin);
-  void icqExportUsers(UinList &, unsigned short);
+  void icqRenameUser(const char *_szId);
+  void icqExportUsers(UserStringList &, unsigned short);
   void icqExportGroups(GroupList &);
+  void icqUpdatePhoneBookTimestamp();
+  void icqUpdatePictureTimestamp();
+  void icqSetPhoneFollowMeStatus(unsigned long nNewStatus);
   void icqUpdateContactList();
+  void icqTypingNotification(const char *_szId, bool _bActive);
 
   // Visible/Invisible/Ignore list functions
-  void icqAddToVisibleList(unsigned long nUin);
-  void icqRemoveFromVisibleList(unsigned long nUin);
-  void icqToggleVisibleList(unsigned long nUin);
-  void icqAddToInvisibleList(unsigned long nUin);
-  void icqRemoveFromInvisibleList(unsigned long nUin);
-  void icqToggleInvisibleList(unsigned long nUin);
-  void icqAddToIgnoreList(unsigned long nUin);
-  void icqRemoveFromIgnoreList(unsigned long nUin);
-  void icqToggleIgnoreList(unsigned long nUin);
+  void icqAddToVisibleList(const char *_szId, unsigned long _nPPID);
+  void icqRemoveFromVisibleList(const char *_szId, unsigned long _nPPID);
+  void icqToggleVisibleList(const char *_szId, unsigned long _nPPID);
+  void icqAddToInvisibleList(const char *_szId, unsigned long _nPPID);
+  void icqRemoveFromInvisibleList(const char *_szId, unsigned long _nPPID);
+  void icqToggleInvisibleList(const char *_szId, unsigned long _nPPID);
+  void icqAddToIgnoreList(const char *_szId, unsigned long _nPPID);
+  void icqRemoveFromIgnoreList(const char *_szId, unsigned long _nPPID);
+  void icqToggleIgnoreList(const char *_szId, unsigned long _nPPID);
+  // Visible/Invisible/Ignore list functions (deprecated versions, for compatibility)
+  void icqAddToVisibleList(unsigned long nUin) __attribute__ ((deprecated)); // deprecated!
+  void icqRemoveFromVisibleList(unsigned long nUin) __attribute__ ((deprecated)); // deprecated!
+  void icqToggleVisibleList(unsigned long nUin) __attribute__ ((deprecated)); // deprecated!
+  void icqAddToInvisibleList(unsigned long nUin) __attribute__ ((deprecated)); // deprecated!
+  void icqRemoveFromInvisibleList(unsigned long nUin) __attribute__ ((deprecated)); // deprecated!
+  void icqToggleInvisibleList(unsigned long nUin) __attribute__ ((deprecated)); // deprecated!
+  void icqAddToIgnoreList(unsigned long nUin) __attribute__ ((deprecated)); // deprecated!
+  void icqRemoveFromIgnoreList(unsigned long nUin) __attribute__ ((deprecated)); // deprecated!
+  void icqToggleIgnoreList(unsigned long nUin) __attribute__ ((deprecated)); // deprecated!
 
   void CheckExport();
-  
+
   void PluginList(PluginsList &l);
   void PluginShutdown(int);
   void PluginEnable(int);
   void PluginDisable(int);
   bool PluginLoad(const char *, int, char **);
 
-#ifdef PROTOCOL_PLUGIN
+
   void ProtoPluginList(ProtoPluginsList &);
+  void ProtoPluginShutdown(unsigned short);
   bool ProtoPluginLoad(const char *);
   int RegisterProtoPlugin();
+  void UnregisterProtoPlugin();
   char *ProtoPluginName(unsigned long);
-#endif
 
-  void PluginUIViewEvent(unsigned long nUin) {
-  	PushPluginSignal(new CICQSignal(SIGNAL_UI_VIEWEVENT, 0, nUin, 0, 0));
+  void PluginUIViewEvent(const char *szId, unsigned long nPPID ) {
+    PushPluginSignal(new CICQSignal(SIGNAL_UI_VIEWEVENT, 0, szId, nPPID, 0, 0));
   }
-  void PluginUIMessage(unsigned long nUin ) {
-  	PushPluginSignal(new CICQSignal(SIGNAL_UI_MESSAGE,0,nUin, 0,0));
+  void PluginUIMessage(const char *szId, unsigned long nPPID) {
+    PushPluginSignal(new CICQSignal(SIGNAL_UI_MESSAGE, 0, szId, nPPID, 0,0));
   }
 
   void UpdateAllUsers();
@@ -294,18 +419,16 @@ public:
   unsigned short BirthdayRange() { return m_nBirthdayRange; }
   void BirthdayRange(unsigned short r) { m_nBirthdayRange = r; }
 
-#ifdef PROTOCOL_PLUGIN
   bool AddUserToList(const char *szId, unsigned long PPID, bool bNotify = true);
-#endif
-
   bool AddUserToList(unsigned long _nUin, bool bNotify = true);
   void AddUserToList(ICQUser *);
   void RemoveUserFromList(unsigned long _nUin);
+  void RemoveUserFromList(const char *szId, unsigned long nPPID);
 
   // SMS
   unsigned long icqSendSms(const char *szNumber, const char *szMessage,
 			   unsigned long nUin);
-  
+
   // NOT MT SAFE
   const char *getUrlViewer();
   void setUrlViewer(const char *s);
@@ -341,12 +464,20 @@ public:
   void SetProxyLogin(const char *s) {  SetString(&m_szProxyLogin, s);  }
   const char *ProxyPasswd() {  return m_szProxyPasswd;  }
   void SetProxyPasswd(const char *s) {  SetString(&m_szProxyPasswd, s);  }
-  
+
   unsigned short TCPPortsLow() { return m_nTCPPortsLow; }
   unsigned short TCPPortsHigh() { return m_nTCPPortsHigh; }
   void SetTCPPorts(unsigned short p, unsigned short r);
   static bool CryptoEnabled();
 
+  bool AutoUpdateInfo()          { return m_bAutoUpdateInfo; }
+  bool AutoUpdateInfoPlugins()   { return m_bAutoUpdateInfoPlugins; }
+  bool AutoUpdateStatusPlugins() { return m_bAutoUpdateStatusPlugins; }
+
+  void SetAutoUpdateInfo(bool b)          { m_bAutoUpdateInfo = b; }
+  void SetAutoUpdateInfoPlugins(bool b)   { m_bAutoUpdateInfoPlugins = b; }
+  void SetAutoUpdateStatusPlugins(bool b) { m_bAutoUpdateStatusPlugins = b; }
+
   const char *Terminal();
   void SetTerminal(const char *s);
   bool Ignore(unsigned short n)      { return m_nIgnoreTypes & n; }
@@ -357,14 +488,19 @@ public:
   void SetAlwaysOnlineNotify(bool);
   CICQSignal *PopPluginSignal();
   ICQEvent *PopPluginEvent();
-#ifdef PROTOCOL_PLUGIN
   CSignal *PopProtoSignal();
-#endif
-  
+
   // Server Side List functions
   bool UseServerContactList()         { return m_bUseSS; }
   void SetUseServerContactList(bool b)  { m_bUseSS = b; }
 
+  // Misc functions
+  bool ReconnectAfterUinClash()              { return m_bReconnectAfterUinClash; }
+  void setReconnectAfterUinClash(bool b)     { m_bReconnectAfterUinClash = b; }
+  bool AddProtocolPlugins();
+  char *parseRTF(const char *);
+
+    
   // Statistics
   CDaemonStats *Stats(unsigned short n) { return n < 3 ? &m_sStats[n] : NULL; }
   DaemonStatsList &AllStats() { return m_sStats; }
@@ -373,12 +509,18 @@ public:
   time_t Uptime() { return time(NULL) - m_nStartTime; }
   void ResetStats();
 
-	// Common message handler
-	void ProcessMessage(ICQUser *user, CBuffer &packet, char *message,
-											unsigned short nMsgType, unsigned long nMask,
-											unsigned long nMsgID[], unsigned long nSequence,
-											bool bIsAck, bool &bNewUser);
+  // Common message handler
+  void ProcessMessage(ICQUser *user, CBuffer &packet, char *message,
+     unsigned short nMsgType, unsigned long nMask,
+     unsigned long nMsgID[], unsigned long nSequence,
+     bool bIsAck, bool &bNewUser);
 
+  bool ProcessPluginMessage(CBuffer &packet, ICQUser *u, unsigned char nChannel,
+     bool bIsAck, unsigned long nMsgID1,
+     unsigned long nMsgID2, unsigned long nSequence,
+     TCPSocket *pSock);
+  bool WaitForReverseConnection(unsigned short id, unsigned long uin);
+  
 protected:
   CLicq *licq;
   COnEventManager m_xOnEventManager;
@@ -392,6 +534,7 @@ protected:
        *m_szRejectFile;
   unsigned long m_nDesiredStatus,
                 m_nIgnoreTypes;
+  bool m_bAutoUpdateInfo, m_bAutoUpdateInfoPlugins, m_bAutoUpdateStatusPlugins;
   unsigned short m_nTCPPortsLow,
                  m_nTCPPortsHigh,
                  m_nMaxUsersPerPacket,
@@ -409,11 +552,13 @@ protected:
        m_bTCPEnabled,
        m_bFirewall;
   time_t m_tLogonTime;
-  
+  char *m_szRegisterPasswd;
+  pthread_t m_nRegisterThreadId;
+
   // ICQ Server
   char *m_szICQServer;
   unsigned short m_nICQServerPort;
-  
+
   // Proxy
   bool m_bProxyEnabled;
   unsigned short m_nProxyType;
@@ -423,14 +568,19 @@ protected:
   char *m_szProxyLogin;
   char *m_szProxyPasswd;
   ProxyServer *m_xProxy;
-  
-  // SS List
-  bool m_bUseSS;
 
+  // Misc
+  bool m_bUseSS; // server side list
+  bool m_bReconnectAfterUinClash; // reconnect after uin has been used from another location?
+  
   // Statistics
   void FlushStats();
   DaemonStatsList m_sStats;
   time_t m_nStartTime, m_nResetTime;
+  
+  static std::list <CReverseConnectToUserData *> m_lReverseConnect;
+  static pthread_mutex_t mutex_reverseconnect;
+  static pthread_cond_t  cond_reverseconnect_done;
 
   std::list <ICQEvent *> m_lxRunningEvents;
   pthread_mutex_t mutex_runningevents;
@@ -443,6 +593,7 @@ protected:
   pthread_mutex_t mutex_cancelthread;
   pthread_t thread_monitorsockets,
             thread_ping,
+            thread_updateusers,
             thread_shutdown;
 
   pthread_cond_t cond_serverack;
@@ -460,7 +611,9 @@ protected:
   void icqSendVisibleList();
   void icqSendInvisibleList();
   void icqRequestSystemMsg();
-  ICQEvent* icqSendThroughServer(unsigned long nUin, unsigned char format, char *_sMessage, CUserEvent* );
+  ICQEvent *icqSendThroughServer(const char *szId, unsigned char format, char *_sMessage,
+    CUserEvent *, unsigned short = 0);
+  ICQEvent* icqSendThroughServer(unsigned long nUin, unsigned char format, char *_sMessage, CUserEvent *, unsigned short = 0);
   void SaveUserList();
 
   void FailEvents(int sd, int err);
@@ -475,15 +628,13 @@ protected:
   void PushExtendedEvent(ICQEvent *);
   void PushPluginSignal(CICQSignal *);
   void PushPluginEvent(ICQEvent *);
-#ifdef PROTOCOL_PLUGIN
   void PushProtoSignal(CSignal *, unsigned long);
-#endif
 
   bool SendEvent(int nSD, CPacket &, bool);
   bool SendEvent(INetSocket *, CPacket &, bool);
   void SendEvent_Server(CPacket *packet);
-  ICQEvent *SendExpectEvent_Server(unsigned long nUin, CPacket *, CUserEvent *,
-                                   bool = false);
+  ICQEvent *SendExpectEvent_Server(unsigned long nUin, CPacket *, CUserEvent *, bool = false);
+  ICQEvent *SendExpectEvent_Server(const char *, unsigned long, CPacket *, CUserEvent *, bool = false);
   ICQEvent *SendExpectEvent_Client(ICQUser *, CPacket *, CUserEvent *);
   ICQEvent *SendExpectEvent(ICQEvent *, void *(*fcn)(void *));
   void AckTCP(CPacketTcp &, int);
@@ -512,18 +663,29 @@ protected:
   void ProcessFifo(char *);
 
   static bool Handshake_Send(TCPSocket *, unsigned long, unsigned short,
-                             unsigned short, bool = true);
-  static bool Handshake_Recv(TCPSocket *, unsigned short, bool = true);
+                             unsigned short, bool = true, unsigned long = 0);
+  static bool Handshake_SendConfirm_v7(TCPSocket *);
+  static bool Handshake_Recv(TCPSocket *, unsigned short, bool = true, bool = false);
+  static bool Handshake_RecvConfirm_v7(TCPSocket *);
   int ConnectToServer(const char* server, unsigned short port);
   int ConnectToLoginServer();
-  int ConnectToUser(unsigned long);
+  int ConnectToUser(unsigned long, unsigned char);
   int ReverseConnectToUser(unsigned long nUin, unsigned long nIp,
-     unsigned short nPort, unsigned short nVersion, unsigned short nFailedPort);
+     unsigned short nPort, unsigned short nVersion, unsigned short nFailedPort,
+     unsigned long nId, unsigned long nMsgID1, unsigned long nMsgID2);
+  int RequestReverseConnection(unsigned long, unsigned long, unsigned long,
+                               unsigned short, unsigned short);
+
+  // Protected plugin related stuff
+  unsigned long icqRequestInfoPlugin(ICQUser *, bool, const char *);
+  unsigned long icqRequestStatusPlugin(ICQUser *, bool, const char *);
+  void icqUpdateInfoTimestamp(const char *);
 
   void StupidChatLinkageFix();
 
   // Declare all our thread functions as friends
   friend void *Ping_tep(void *p);
+  friend void *UpdateUsers_tep(void *p);
   friend void *MonitorSockets_tep(void *p);
   friend void *ReverseConnectToUser_tep(void *p);
   friend void *ProcessRunningEvent_Client_tep(void *p);
@@ -535,8 +697,9 @@ protected:
   friend class CChatManager;
   friend class CFileTransferManager;
   friend class COnEventManager;
-	friend class CUserManager;
+  friend class CUserManager;
   friend class CLicq;
+  friend class CMSN;
 };
 
 // Global pointer
@@ -551,16 +714,25 @@ unsigned short VersionToUse(unsigned short);
 class CReverseConnectToUserData
 {
 public:
-  CReverseConnectToUserData(unsigned long uin, unsigned long ip,
-   unsigned short port, unsigned short version, unsigned short failedport) :
-   nUin(uin), nIp(ip), nPort(port), nFailedPort(failedport),
-   nVersion(version) {}
+  CReverseConnectToUserData(unsigned long uin, unsigned long id,
+   unsigned long data, unsigned long ip, unsigned short port,
+   unsigned short version, unsigned short failedport, unsigned long msgid1,
+   unsigned long msgid2) :
+   nUin(uin), nId(id), nData(data), nIp(ip), nPort(port),
+   nFailedPort(failedport), nVersion(version), nMsgID1(msgid1),
+   nMsgID2(msgid2), bSuccess(false), bFinished(false) {}
 
   unsigned long nUin;
+  unsigned long nId;
+  unsigned long nData;
   unsigned long nIp;
   unsigned short nPort;
   unsigned short nFailedPort;
   unsigned short nVersion;
+  unsigned long nMsgID1;
+  unsigned long nMsgID2;
+  bool bSuccess;
+  bool bFinished;
 };
 
 
