@@ -3,12 +3,15 @@
 
 #include <ctype.h>
 
-const unsigned short NUM_COMMANDS = 23;
+const unsigned short NUM_COMMANDS = 24;
 const struct SCommand aCommands[NUM_COMMANDS] =
 {
   { "contacts", &CLicqConsole::MenuContactList, NULL,
     " %B%cco%bntacts",
     "Force a refresh of the contact list." },
+  { "console", &CLicqConsole::MenuSwitchConsole, NULL,
+	" %B%ccons%bole <num>",
+	"Switch to a console." },
   { "group", &CLicqConsole::MenuGroup, NULL,
     " %B%cg%broup [ %B#%b ]",
     "Prints the group list or changes to the given group number." },
@@ -91,7 +94,7 @@ const struct SCommand aCommands[NUM_COMMANDS] =
     "A macro can be any string of characters not containing\n"
     "a space.  The command can be any valid command, do not\n"
     "prepend the command character when defining.\n"
-    "Example: \"/define r last message\" creates a macro \"r\"\n"
+    "Example: \"/define r message $\" creates a macro \"r\"\n"
     "which replies to the last user you talked to."},
   { "help", &CLicqConsole::MenuHelp, NULL,
     " %B%che%blp [ %B<command>%b ]",
@@ -102,6 +105,110 @@ const struct SCommand aCommands[NUM_COMMANDS] =
     "Quit Licq." }
 };
 
+/*---------------------------------------------------------------------------
+ * CLicqConsole::MenuPopupWrapper
+ *
+ * Callback function for when SPACE is pressed on the cdkUserList
+ *-------------------------------------------------------------------------*/
+int CLicqConsole::MenuPopupWrapper(EObjectType cdktype, void *object, void *clientData, chtype key)
+{
+  CLicqConsole *me = (CLicqConsole *)clientData;
+  me->MenuPopup(me->cdkUserList->currentItem);
+  return 1;
+}
+
+/*---------------------------------------------------------------------------
+ * CLicqConsole::MenuPopup
+ *-------------------------------------------------------------------------*/
+void CLicqConsole::MenuPopup(int userSelected) {
+  list <SScrollUser *>::iterator it;
+  for (it = m_lScrollUsers.begin(); it != m_lScrollUsers.end(); it++)
+  {
+    if ((*it)->pos == userSelected)
+    {
+      int choice;
+      ICQUser *u = gUserManager.FetchUser((*it)->szId, (*it)->nPPID, LOCK_R);
+      if (u == NULL) return;
+
+      PrintContactPopup(u->GetAlias());
+      gUserManager.DropUser(u);
+      nl();
+      choice = activateCDKScroll(cdkContactPopup, NULL);
+      eraseCDKScroll(cdkContactPopup);
+      destroyCDKScroll(cdkContactPopup);
+      winMain->RefreshWin();
+      if (cdkContactPopup->exitType == vNORMAL)
+      {
+        nonl();
+        switch (choice)
+        {
+          case 0:
+            UserCommand_Msg((*it)->szId, (*it)->nPPID, NULL);
+            break;
+          case 1:
+            UserCommand_View((*it)->szId, (*it)->nPPID, NULL);
+            break;
+        }
+	  }
+	  break;
+    }
+  }
+}
+
+/*---------------------------------------------------------------------------
+ * CLicqConsole::MenuSwitchConsole
+ *-------------------------------------------------------------------------*/
+void CLicqConsole::MenuSwitchConsole(char *_szArg)
+{
+  int i;
+  if (!_szArg)
+    return;
+  i = atoi(_szArg);
+  if (i <= 0 || i > 9)
+    winMain->wprintf("Invalid console number - valid numbers are 1-9\n");
+  else
+    if (i == 9)
+      SwitchToCon(0); 
+    else
+      SwitchToCon(i);
+}
+
+/*---------------------------------------------------------------------------
+ * CLicqConsole::MenuList
+ *-------------------------------------------------------------------------*/
+void CLicqConsole::MenuList(char *_szArg)
+{
+  int userSelected;
+  UserListHighlight(A_REVERSE);
+  nl();
+  userSelected = activateCDKScroll(cdkUserList, NULL);
+  nonl();
+  UserListHighlight(A_NORMAL);
+  drawCDKScroll(cdkUserList, TRUE);
+  if (cdkUserList->exitType == vNORMAL)
+  {
+    list <SScrollUser *>::iterator it;
+	for (it = m_lScrollUsers.begin(); it != m_lScrollUsers.end(); it++)
+    {
+      if ((*it)->pos == userSelected)
+      {
+        ICQUser *u = gUserManager.FetchUser((*it)->szId, (*it)->nPPID, LOCK_R);
+        if (u == NULL) return;
+        if (u->NewMessages() > 0)
+        {
+          gUserManager.DropUser(u);
+          UserCommand_View((*it)->szId, (*it)->nPPID, NULL);
+        }
+        else
+        {
+          gUserManager.DropUser(u);
+          UserCommand_Msg((*it)->szId, (*it)->nPPID, NULL);
+        }
+        break;
+      }
+    }
+  }
+}
 
 
 /*---------------------------------------------------------------------------
@@ -150,7 +257,10 @@ void CLicqConsole::MenuPlugins(char *_szArg)
   PluginsList l;
   PluginsListIter it;
   licqDaemon->PluginList(l);
-  PrintBoxTop("Plugins", COLOR_BLUE, 70);
+  ProtoPluginsList p1;
+  ProtoPluginsListIter pit;
+  licqDaemon->ProtoPluginList(p1);
+  PrintBoxTop("Plugins", 40, 70);
   for (it = l.begin(); it != l.end(); it++)
   {
     PrintBoxLeft();
@@ -159,6 +269,13 @@ void CLicqConsole::MenuPlugins(char *_szArg)
                      (*it)->BuildTime(), (*it)->Status());
     PrintBoxRight(70);
   }
+  for (pit = p1.begin(); pit != p1.end(); pit++)
+  {
+    PrintBoxLeft();
+    winMain->wprintf("[%3d] %s v%s", (*pit)->Id(), (*pit)->Name(),
+		     (*pit)->Version());
+    PrintBoxRight(70);
+  }
   PrintBoxBottom(70);
 }
 
@@ -196,7 +313,7 @@ void CLicqConsole::MenuDefine(char *szArg)
         return;
       }
     }
-    winMain->wprintf("%CNo such macro \"%A%s%Z\"\n", COLOR_RED, A_BOLD,
+    winMain->wprintf("%CNo such macro \"%A%s%Z\"\n", 16, A_BOLD,
       szArg, A_BOLD);
     return;
   }
@@ -254,7 +371,7 @@ void CLicqConsole::MenuGroup(char *_szArg)
 
     if (nCurrentGroup > NUM_GROUPS_SYSTEM || nCurrentGroup == 0)
     {
-      winMain->wprintf("%CInvalid group number (0 - %d)\n", COLOR_RED,
+      winMain->wprintf("%CInvalid group number (0 - %d)\n", 16,
                        NUM_GROUPS_SYSTEM);
       return;
     }
@@ -272,7 +389,7 @@ void CLicqConsole::MenuGroup(char *_szArg)
 
     if (nCurrentGroup > gUserManager.NumGroups())
     {
-      winMain->wprintf("%CInvalid group number (0 - %d)\n", COLOR_RED,
+      winMain->wprintf("%CInvalid group number (0 - %d)\n", 16,
                        gUserManager.NumGroups());
       return;
     }
@@ -295,16 +412,16 @@ void CLicqConsole::MenuGroup(char *_szArg)
 /*---------------------------------------------------------------------------
  * CLicqConsole::MenuAdd
  *-------------------------------------------------------------------------*/
+//TODO: fix this for other protocols
 void CLicqConsole::MenuAdd(char *szArg)
 {
   if (szArg == NULL)
   {
-    winMain->wprintf("%CSpecify a UIN to add.\n", COLOR_RED);
+    winMain->wprintf("%CSpecify a UIN to add.\n", 16);
     return;
   }
 
   // Try to change groups
-  int nUin = atol(szArg);
   bool bAlert = false;
 
   while (*szArg != '\0' && *szArg != ' ') szArg++;
@@ -315,19 +432,20 @@ void CLicqConsole::MenuAdd(char *szArg)
       bAlert = true;
   }
 
-  if (!licqDaemon->AddUserToList(nUin))
+  if (!licqDaemon->AddUserToList(szArg, LICQ_PPID))
   {
-    winMain->wprintf("%CAdding user %lu failed (duplicate user or invalid uin).\n",
-     COLOR_RED, nUin);
+    winMain->wprintf("%CAdding user %s failed (duplicate user or invalid uin).\n",
+     16, szArg);
     return;
   }
 
-  winMain->wprintf("%C%AAdded user %ld.\n",
+  winMain->wprintf("%C%AAdded user %s.\n",
                      m_cColorInfo->nColor, m_cColorInfo->nAttr,
-                     nUin);
+                     szArg);
 
   if (bAlert)
   {
+    int nUin = atol(szArg);
     licqDaemon->icqAlertUser(nUin);
     winMain->wprintf("%C%AAlerted user %ld they were added.\n",
                      m_cColorInfo->nColor, m_cColorInfo->nAttr,
@@ -344,7 +462,7 @@ void CLicqConsole::MenuAuthorize(char *szArg)
 {
   if (szArg == NULL)
   {
-    winMain->wprintf("%CSpecify \"grant/refuse\" and a UIN to authorize.\n", COLOR_RED);
+    winMain->wprintf("%CSpecify \"grant/refuse\" and a UIN/Screen Name to authorize.\n", 16);
     return;
   }
 
@@ -361,19 +479,10 @@ void CLicqConsole::MenuAuthorize(char *szArg)
     szArg += 6;
   }
 
-  // Try to change groups
-  int nUin = atol(szArg);
-
-  if (nUin == 0)
-  {
-    winMain->wprintf("%CUIN must be non-zero.\n", COLOR_RED);
-    return;
-  }
-
   // Get the input now
   winMain->fProcessInput = &CLicqConsole::InputAuthorize;
   winMain->state = STATE_MLE;
-  DataMsg *data = new DataMsg(nUin);
+  DataMsg *data = new DataMsg(szArg, LICQ_PPID);
   data->bUrgent = bGrant;
   winMain->data = data;
 
@@ -381,8 +490,6 @@ void CLicqConsole::MenuAuthorize(char *szArg)
                    m_cColorQuery->nAttr, m_cColorQuery->nColor);
 
   return;
-
-
 }
 
 
@@ -396,7 +503,7 @@ void CLicqConsole::MenuStatus(char *_szArg)
 
   if (_szArg == NULL)
   {
-    winMain->wprintf("%CSpecify status.\n", COLOR_RED);
+    winMain->wprintf("%CSpecify status.\n", 16);
     return;
   }
 
@@ -418,27 +525,39 @@ void CLicqConsole::MenuStatus(char *_szArg)
   // Check that we found it
   if (i == NUM_STATUS)
   {
-    winMain->wprintf("%CInvalid status: %A%s\n", COLOR_RED, A_BOLD, _szArg);
+    winMain->wprintf("%CInvalid status: %A%s\n", 16, A_BOLD, _szArg);
     return;
   }
 
-  ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
-  if (nStatus == ICQ_STATUS_OFFLINE)
+  //set same status for all protocols for now
+  ProtoPluginsList p1;
+  ProtoPluginsListIter it;
+  licqDaemon->ProtoPluginList(p1);
+  for (it = p1.begin(); it != p1.end(); it++)
   {
-    gUserManager.DropOwner();
-    licqDaemon->icqLogoff();
-    return;
-  }
-  if (bInvisible)
-    nStatus |= ICQ_STATUS_FxPRIVATE;
+    unsigned long nPPID = (*it)->PPID();
+    ICQOwner *o = gUserManager.FetchOwner(nPPID, LOCK_R);
+    if (nStatus == ICQ_STATUS_OFFLINE)
+    {
+      gUserManager.DropOwner(nPPID);
+      licqDaemon->ProtoLogoff(nPPID);
+      continue;
+    }
+    if (bInvisible)
+      nStatus |= ICQ_STATUS_FxPRIVATE;
 
-  // call the right function
-  bool b = o->StatusOffline();
-  gUserManager.DropOwner();
-  if (b)
-     licqDaemon->icqLogon(nStatus);
-  else
-     licqDaemon->icqSetStatus(nStatus);
+    // call the right function
+    bool b = o->StatusOffline();
+    gUserManager.DropOwner(nPPID);
+    if (b)
+    {
+       licqDaemon->ProtoLogon(nPPID, nStatus);
+    }
+    else
+    {
+       licqDaemon->ProtoSetStatus(nPPID, nStatus);
+    }
+  }
 }
 
 
@@ -462,15 +581,13 @@ void CLicqConsole::MenuUins(char *)
 
   for (it = m_lUsers.begin(); it != m_lUsers.end(); it++)
   {
-    u = gUserManager.FetchUser((*it)->nUin, LOCK_R);
-    winMain->wprintf("%s %A-%Z %lu\n", u->GetAlias(), A_BOLD, A_BOLD, u->Uin());
+    u = gUserManager.FetchUser((*it)->szId, (*it)->nPPID, LOCK_R);
+    winMain->wprintf("%s %A-%Z %s\n", u->GetAlias(), A_BOLD, A_BOLD, u->IdString());
     gUserManager.DropUser(u);
   }
 
 }
 
-
-
 /*---------------------------------------------------------------------------
  * CLicqConsole::GetUinFromArg
  *-------------------------------------------------------------------------*/
@@ -493,7 +610,7 @@ unsigned long CLicqConsole::GetUinFromArg(char **p_szArg)
     szCmd = strchr(&szArg[1], '"');
     if (szCmd == NULL)
     {
-      winMain->wprintf("%CUnbalanced quotes.\n", COLOR_RED);
+      winMain->wprintf("%CUnbalanced quotes.\n", 16);
       return (unsigned long)-1;
     }
     *szCmd++ = '\0';
@@ -533,7 +650,7 @@ unsigned long CLicqConsole::GetUinFromArg(char **p_szArg)
 
   if (nUin == 0)
   {
-    FOR_EACH_USER_START(LOCK_R)
+    FOR_EACH_PROTO_USER_START(LICQ_PPID, LOCK_R)
     {
       if (strcasecmp(szAlias, pUser->GetAlias()) == 0)
       {
@@ -541,10 +658,10 @@ unsigned long CLicqConsole::GetUinFromArg(char **p_szArg)
         FOR_EACH_USER_BREAK;
       }
     }
-    FOR_EACH_USER_END
+    FOR_EACH_PROTO_USER_END
     if (nUin == 0)
     {
-      winMain->wprintf("%CInvalid user: %A%s\n", COLOR_RED, A_BOLD, szAlias);
+      winMain->wprintf("%CInvalid user: %A%s\n", 16, A_BOLD, szAlias);
       return (unsigned long)-1;
     }
   }
@@ -552,7 +669,7 @@ unsigned long CLicqConsole::GetUinFromArg(char **p_szArg)
   {
     if (!gUserManager.IsOnList(nUin))
     {
-      winMain->wprintf("%CInvalid uin: %A%lu\n", COLOR_RED, A_BOLD, nUin);
+      winMain->wprintf("%CInvalid uin: %A%lu\n", 16, A_BOLD, nUin);
       return (unsigned long)-1;
     }
   }
@@ -567,6 +684,93 @@ unsigned long CLicqConsole::GetUinFromArg(char **p_szArg)
   return nUin;
 }
 
+/*---------------------------------------------------------------------------
+ * CLicqConsole::GetContactFromArg
+ *-------------------------------------------------------------------------*/
+struct SContact CLicqConsole::GetContactFromArg(char **p_szArg)
+{
+  char *szAlias, *szCmd;
+  char *szArg = *p_szArg;
+  struct SContact scon;
+  scon.szId = NULL;
+
+  if (szArg == NULL) {
+    return scon;
+  }
+
+  // Check if the alias is quoted
+  if (szArg[0] == '"')
+  {
+    szAlias = &szArg[1];
+    szCmd = strchr(&szArg[1], '"');
+    if (szCmd == NULL)
+    {
+      winMain->wprintf("%CUnbalanced quotes.\n", 16);
+      return scon;
+    }
+    *szCmd++ = '\0';
+    szCmd = strchr(szCmd, ' ');
+  }
+  else if (szArg[0] == '#')
+  {
+    *p_szArg = NULL;
+    ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+    scon.szId = o->IdString();
+    scon.nPPID = o->PPID(); 
+    gUserManager.DropOwner();
+    return scon;
+  }
+  else if (szArg[0] == '$')
+  {
+    *p_szArg = NULL;
+    return winMain->sLastContact;
+  }
+  else
+  {
+    szAlias = szArg;
+    szCmd = strchr(szArg, ' ');
+  }
+
+  if (szCmd != NULL)
+  {
+    *szCmd++ = '\0';
+    STRIP(szCmd);
+  }
+  *p_szArg = szCmd;
+
+  FOR_EACH_USER_START(LOCK_R)
+  {
+    if (strcasecmp(szAlias, pUser->GetAlias()) == 0)
+    {
+      scon.szId = pUser->IdString();
+      scon.nPPID = pUser->PPID();
+      FOR_EACH_PROTO_USER_BREAK;
+    }
+  }
+  FOR_EACH_USER_END
+  if (scon.szId == NULL)
+  {
+    winMain->wprintf("%CInvalid user: %A%s\n", 16, A_BOLD, szAlias);
+    scon.szId = NULL;
+    scon.nPPID = (unsigned long)-1;
+    return scon;
+  }
+ 
+  // Save this as the last user
+  if (winMain->sLastContact.szId == 0 ||
+      !(strcmp(scon.szId, winMain->sLastContact.szId) == 0 &&
+      scon.nPPID == winMain->sLastContact.nPPID))
+  {
+    if (winMain->sLastContact.szId)
+      free(winMain->sLastContact.szId);
+    winMain->sLastContact.nPPID = scon.nPPID;
+    winMain->sLastContact.szId = strdup(scon.szId);
+    PrintStatus();
+  }
+
+  return scon;
+}
+
 
 
 /*---------------------------------------------------------------------------
@@ -575,63 +779,72 @@ unsigned long CLicqConsole::GetUinFromArg(char **p_szArg)
 void CLicqConsole::MenuMessage(char *szArg)
 {
   char *sz = szArg;
-  unsigned long nUin = GetUinFromArg(&sz);
+  struct SContact scon = GetContactFromArg(&sz);
 
-  if (nUin == gUserManager.OwnerUin())
-    winMain->wprintf("%CYou can't send messages to yourself!\n", COLOR_RED);
-  else if (nUin == 0)
-    winMain->wprintf("%CYou must specify a user to send a message to.\n", COLOR_RED);
-  else if (nUin != (unsigned long)-1)
-    UserCommand_Msg(nUin, sz);
+  if (!scon.szId && scon.nPPID != (unsigned long)-1)
+    winMain->wprintf("%CYou must specify a user to send a message to.\n", 16);
+  else if (scon.nPPID != (unsigned long)-1)
+    UserCommand_Msg(scon.szId, scon.nPPID, sz);
 }
 
 
 /*---------------------------------------------------------------------------
  * CLicqConsole::MenuInfo
  *-------------------------------------------------------------------------*/
+//TODO: fix this for other protocols
 void CLicqConsole::MenuInfo(char *szArg)
 {
   char *sz = szArg;
   unsigned long nUin = GetUinFromArg(&sz);
 
   if (nUin == gUserManager.OwnerUin())
-    winMain->wprintf("%CSetting personal info not implemented yet.\n", COLOR_RED);
-  else if (nUin == 0)
-    UserCommand_Info(gUserManager.OwnerUin(), sz);
-  else if (nUin != (unsigned long)-1)
-    UserCommand_Info(nUin, sz);
+    winMain->wprintf("%CSetting personal info not implemented yet.\n", 16);
+  else if (nUin == 0) {
+    char szUin[24];
+    sprintf(szArg, "%lu", gUserManager.OwnerUin());
+    UserCommand_Info(szUin, LICQ_PPID, sz);
+  } else if (nUin != (unsigned long)-1)
+    sprintf(szArg, "%lu", nUin);
+    UserCommand_Info(szArg, LICQ_PPID, sz);
 }
 
 
 /*---------------------------------------------------------------------------
  * CLicqConsole::MenuUrl
  *-------------------------------------------------------------------------*/
+//TODO: fix this for other protocols
 void CLicqConsole::MenuUrl(char *szArg)
 {
   char *sz = szArg;
   unsigned long nUin = GetUinFromArg(&sz);
 
   if (nUin == gUserManager.OwnerUin())
-    winMain->wprintf("%CYou can't send URLs to yourself!\n", COLOR_RED);
+    winMain->wprintf("%CYou can't send URLs to yourself!\n", 16);
   else if (nUin == 0)
-    winMain->wprintf("%CYou must specify a user to send a URL to.\n", COLOR_RED);
+    winMain->wprintf("%CYou must specify a user to send a URL to.\n", 16);
   else if (nUin != (unsigned long)-1)
-    UserCommand_Url(nUin, sz);
+    sprintf(szArg, "%lu", nUin);
+    UserCommand_Url(szArg, LICQ_PPID, sz);
+
 }
 
 
 /*---------------------------------------------------------------------------
  * CLicqConsole::MenuSms
  *-------------------------------------------------------------------------*/
+//TODO: fix this for other protocols
 void CLicqConsole::MenuSms(char *szArg)
 {
   char *sz = szArg;
   unsigned long nUin = GetUinFromArg(&sz);
 
   if (nUin == 0)
-    winMain->wprintf("%CInvalid user\n", COLOR_RED);
+    winMain->wprintf("%CInvalid user\n", 16);
   else if (nUin != (unsigned long)-1)
-    UserCommand_Sms(nUin, sz);   
+  {
+    sprintf(szArg, "%lu", nUin);
+    UserCommand_Sms(szArg, LICQ_PPID, sz);
+  }
 }
 
 
@@ -640,21 +853,27 @@ void CLicqConsole::MenuSms(char *szArg)
  *-------------------------------------------------------------------------*/
 void CLicqConsole::MenuView(char *szArg)
 {
-  char *sz = szArg;
-  unsigned long nUin = GetUinFromArg(&sz);
 
-  if (nUin == 0)
+  char *sz = szArg;
+  char *szId = NULL;
+  unsigned long nPPID = (unsigned long)-1;
+  struct SContact scon = GetContactFromArg(&sz);
+  
+  if (scon.szId == 0)
   {
     // Do nothing if there are no events pending
     if (ICQUser::getNumUserEvents() == 0) return;
 
-    // Do system messages first
-    ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+    // Do icq system messages first
+    ICQOwner *o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
     unsigned short nNumMsg = o->NewMessages();
     gUserManager.DropOwner();
     if (nNumMsg > 0)
     {
-      UserCommand_View(gUserManager.OwnerUin(), NULL);
+      //TODO which owner?
+      char szUin[24];
+      sprintf(szUin, "%lu", gUserManager.OwnerUin());
+      UserCommand_View(szUin, LICQ_PPID, NULL);
       return;
     }
 
@@ -663,17 +882,22 @@ void CLicqConsole::MenuView(char *szArg)
     {
       if (pUser->NewMessages() > 0 && pUser->Touched() <= t)
       {
-        nUin = pUser->Uin();
+        szId = pUser->IdString();
+	nPPID = pUser->PPID();
         t = pUser->Touched();
       }
     }
     FOR_EACH_USER_END
-    if (nUin != 0) UserCommand_View(nUin, NULL);
+    if (szId != NULL)
+    {
+      UserCommand_View(szId, nPPID, NULL);
+    }
   }
-  else if (nUin != (unsigned long)-1)
+  else if (scon.szId != NULL)
   {
-    UserCommand_View(nUin, sz);
+    UserCommand_View(scon.szId, scon.nPPID, sz);
   }
+
 }
 
 
@@ -686,11 +910,14 @@ void CLicqConsole::MenuSecure(char *szArg)
   unsigned long nUin = GetUinFromArg(&sz);
 
   if (nUin == gUserManager.OwnerUin())
-    winMain->wprintf("%CYou can't establish a secure connection to yourself!\n", COLOR_RED);
+    winMain->wprintf("%CYou can't establish a secure connection to yourself!\n", 16);
   else if (nUin == 0)
-    winMain->wprintf("%CYou must specify a user to talk to.\n", COLOR_RED);
+    winMain->wprintf("%CYou must specify a user to talk to.\n", 16);
   else if (nUin != (unsigned long)-1)
-    UserCommand_Secure(nUin, sz);
+  {
+    sprintf(szArg, "%lu", nUin);
+    UserCommand_Secure(szArg, LICQ_PPID, sz);
+  }
 }
 
 
@@ -703,7 +930,7 @@ void CLicqConsole::MenuFile(char *szArg)
   unsigned long nUin = GetUinFromArg(&sz);
 
   if (nUin == gUserManager.OwnerUin())
-    winMain->wprintf("%CYou can't send files to yourself!\n", COLOR_RED);
+    winMain->wprintf("%CYou can't send files to yourself!\n", 16);
   else if (nUin == 0)
   {
     bool bNum = false;
@@ -724,7 +951,10 @@ void CLicqConsole::MenuFile(char *szArg)
     }
   }
   else if (nUin != (unsigned long)-1)
-    UserCommand_SendFile(nUin, sz);
+  {
+    sprintf(szArg, "%lu", nUin);
+    UserCommand_SendFile(szArg, LICQ_PPID, sz);
+  }
 }
 
 
@@ -742,9 +972,9 @@ void CLicqConsole::MenuAutoResponse(char *szArg)
     for (unsigned short i = 0; i < winMain->Cols() - 10; i++)
       waddch(winMain->Win(), ACS_HLINE);
     waddch(winMain->Win(), '\n');
-    ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+    ICQOwner *o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
     winMain->wprintf("%B%CAuto response:\n%b%s\n",
-                     COLOR_WHITE, o->AutoResponse());
+                     8, o->AutoResponse());
     gUserManager.DropOwner();
     wattron(winMain->Win(), A_BOLD);
     for (unsigned short i = 0; i < winMain->Cols() - 10; i++)
@@ -754,26 +984,30 @@ void CLicqConsole::MenuAutoResponse(char *szArg)
     wattroff(winMain->Win(), A_BOLD);
   }
   else if (nUin == 0)
-    UserCommand_SetAutoResponse(nUin, sz);
+    UserCommand_SetAutoResponse(NULL, LICQ_PPID, sz);
   else if (nUin != (unsigned long)-1)
-    UserCommand_FetchAutoResponse(nUin, sz);
+    UserCommand_FetchAutoResponse(NULL, LICQ_PPID, sz);
 }
 
 
 /*---------------------------------------------------------------------------
  * CLicqConsole::MenuRemove
  *-------------------------------------------------------------------------*/
+//TODO: add support for other protocols
 void CLicqConsole::MenuRemove(char *szArg)
 {
   char *sz = szArg;
   unsigned long nUin = GetUinFromArg(&sz);
 
   if (nUin == gUserManager.OwnerUin())
-    winMain->wprintf("%CYou can't remove yourself!\n", COLOR_RED);
+    winMain->wprintf("%CYou can't remove yourself!\n", 16);
   else if (nUin == 0)
-    winMain->wprintf("%CYou must specify a user to remove.\n", COLOR_RED);
+    winMain->wprintf("%CYou must specify a user to remove.\n", 16);
   else if (nUin != (unsigned long)-1)
-    UserCommand_Remove(nUin, sz);
+  {
+    sprintf(szArg, "%lu", nUin);
+    UserCommand_Remove(szArg, LICQ_PPID, sz);
+  }
 }
 
 
@@ -783,12 +1017,12 @@ void CLicqConsole::MenuRemove(char *szArg)
 void CLicqConsole::MenuHistory(char *szArg)
 {
   char *sz = szArg;
-  unsigned long nUin = GetUinFromArg(&sz);
+  struct SContact scon = GetContactFromArg(&sz);
 
-  if (nUin == 0)
-    winMain->wprintf("%CYou must specify a user to view history.\n", COLOR_RED);
-  else if (nUin != (unsigned long)-1)
-    UserCommand_History(nUin, sz);
+  if (!scon.szId && scon.nPPID != (unsigned long)-1)
+    winMain->wprintf("%CYou must specify a user to view history.\n", 16);
+  else if (scon.nPPID != (unsigned long)-1)
+    UserCommand_History(scon.szId, scon.nPPID, sz);
 }
 
 
@@ -828,7 +1062,7 @@ void CLicqConsole::MenuSet(char *_szArg)
   }
   if (i == NUM_VARIABLES)
   {
-    winMain->wprintf("%CNo such variable: %A%s\n", COLOR_RED, A_BOLD, szVariable);
+    winMain->wprintf("%CNo such variable: %A%s\n", 16, A_BOLD, szVariable);
     return;
   }
 
@@ -858,7 +1092,7 @@ void CLicqConsole::MenuSet(char *_szArg)
     }
     if (i == NUM_COLORMAPS)
     {
-      winMain->wprintf("%CNo such color: %A%s\n", COLOR_RED, A_BOLD, szValue);
+      winMain->wprintf("%CNo such color: %A%s\n", 16, A_BOLD, szValue);
       break;
     }
 
@@ -881,7 +1115,7 @@ void CLicqConsole::MenuSet(char *_szArg)
   case STRING:
     if (szValue[0] != '"' || szValue[strlen(szValue) - 1] != '"')
     {
-      winMain->wprintf("%CString values must be enclosed by double quotes (\").\n", COLOR_RED);
+      winMain->wprintf("%CString values must be enclosed by double quotes (\").\n", 16);
       return;
     }
     szValue[strlen(szValue) - 1] = '\0';
