@@ -232,11 +232,20 @@ void OptionsDlg::SetupOptions()
       cmbDockTheme->setEnabled(false);
       rdbDockDefault->setEnabled(false);
       rdbDockThemed->setEnabled(false);
+      rdbDockSmall->setChecked(false);
+      rdbDockSmall->setEnabled(false);
       chkDockFortyEight->setEnabled(false);
       chkHidden->setEnabled(false);
       chkHidden->setChecked(false);
       break;
 #ifndef USE_KDE
+    case DockSmall:
+      chkUseDock->setChecked(true);
+      rdbDockSmall->setChecked(true);
+      chkDockFortyEight->setEnabled(false);
+      cmbDockTheme->setEnabled(false);
+      chkHidden->setEnabled(true);
+      break;
     case DockDefault:
       chkUseDock->setChecked(true);
       rdbDockDefault->setChecked(true);
@@ -261,12 +270,15 @@ void OptionsDlg::SetupOptions()
       }
       break;
 #else
+    case DockSmall:
     case DockDefault:
     case DockThemed:
       chkUseDock->setChecked(true);
       cmbDockTheme->setEnabled(false);
       rdbDockDefault->setEnabled(false);
       rdbDockThemed->setEnabled(false);
+      rdbDockSmall->setChecked(true);
+      rdbDockSmall->setEnabled(false);
       chkDockFortyEight->setEnabled(false);
       chkHidden->setEnabled(true);
       break;
@@ -302,6 +314,8 @@ void OptionsDlg::SetupOptions()
   chkProxyAuthEnabled->setChecked(mainwin->licqDaemon->ProxyAuthEnabled());
   edtProxyLogin->setText(QString(mainwin->licqDaemon->ProxyLogin()));
   edtProxyPasswd->setText(QString(mainwin->licqDaemon->ProxyPasswd()));
+  
+  chkReconnectAfterUinClash->setChecked(mainwin->licqDaemon->ReconnectAfterUinClash());
 
   if (!mainwin->licqDaemon->ProxyEnabled())
   {
@@ -329,6 +343,12 @@ void OptionsDlg::SetupOptions()
   chkIgnoreWebPanel->setChecked(mainwin->licqDaemon->Ignore(IGNORE_WEBPANEL));
   chkIgnoreEmailPager->setChecked(mainwin->licqDaemon->Ignore(IGNORE_EMAILPAGER));
 
+  chkAutoUpdateInfo->setChecked(mainwin->licqDaemon->AutoUpdateInfo());
+  chkAutoUpdateInfoPlugins->setChecked(mainwin->licqDaemon
+                                              ->AutoUpdateInfoPlugins());
+  chkAutoUpdateStatusPlugins->setChecked(mainwin->licqDaemon
+                                                ->AutoUpdateStatusPlugins());
+
   // plugins tab
 #if QT_VERSION > 300
   cmbUrlViewer->setCurrentText(mainwin->licqDaemon->getUrlViewer() == NULL ?
@@ -402,11 +422,14 @@ void OptionsDlg::SetupOptions()
    oem->Unlock();
    //TODO make general for all plugins
    ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
-   chkOEAway->setChecked(o->AcceptInAway());
-   chkOENA->setChecked(o->AcceptInNA());
-   chkOEOccupied->setChecked(o->AcceptInOccupied());
-   chkOEDND->setChecked(o->AcceptInDND());
-   gUserManager.DropOwner();
+   if (o)
+   {
+     chkOEAway->setChecked(o->AcceptInAway());
+     chkOENA->setChecked(o->AcceptInNA());
+     chkOEOccupied->setChecked(o->AcceptInOccupied());
+     chkOEDND->setChecked(o->AcceptInDND());
+     gUserManager.DropOwner();
+   }
    chkAlwaysOnlineNotify->setChecked(mainwin->licqDaemon->AlwaysOnlineNotify());
 }
 
@@ -484,7 +507,8 @@ void OptionsDlg::ApplyOptions()
 
 #ifndef USE_KDE
   if (chkUseDock->isChecked() &&
-      (rdbDockDefault->isChecked() || rdbDockThemed->isChecked()) )
+      (rdbDockDefault->isChecked() || rdbDockThemed->isChecked() ||
+       rdbDockSmall->isChecked()) )
 #else
   if (chkUseDock->isChecked())
 #endif
@@ -495,6 +519,7 @@ void OptionsDlg::ApplyOptions()
       if (mainwin->m_nDockMode != DockDefault ||
           ((IconManager_Default *)mainwin->licqIcon)->FortyEight() != chkDockFortyEight->isChecked() )
       {
+        mainwin->licqIcon->close();
         delete mainwin->licqIcon;
         mainwin->licqIcon = new IconManager_Default(mainwin, mainwin->mnuSystem, chkDockFortyEight->isChecked());
         mainwin->m_nDockMode = DockDefault;
@@ -504,7 +529,11 @@ void OptionsDlg::ApplyOptions()
     {
       if (mainwin->m_nDockMode != DockThemed)
       {
-        delete mainwin->licqIcon;
+        if (mainwin->licqIcon)
+        {
+          mainwin->licqIcon->close();
+          delete mainwin->licqIcon; 
+        }
         mainwin->licqIcon = new IconManager_Themed(mainwin, mainwin->mnuSystem, cmbDockTheme->currentText().local8Bit());
         mainwin->m_nDockMode = DockThemed;
       }
@@ -513,6 +542,19 @@ void OptionsDlg::ApplyOptions()
         ((IconManager_Themed *)mainwin->licqIcon)->SetTheme(cmbDockTheme->currentText().local8Bit());
       }
     }
+    else if (rdbDockSmall->isChecked())
+    {
+      if (mainwin->m_nDockMode != DockSmall)
+      {
+        if (mainwin->licqIcon)
+        {
+	  mainwin->licqIcon->close();
+          delete mainwin->licqIcon;
+        }
+        mainwin->licqIcon = new IconManager_KDEStyle(mainwin, mainwin->mnuSystem);
+        mainwin->m_nDockMode = DockSmall;
+      }
+    }
 #else
     if(!mainwin->licqIcon)
       mainwin->licqIcon = new IconManager_KDEStyle(mainwin, mainwin->mnuSystem);
@@ -523,7 +565,8 @@ void OptionsDlg::ApplyOptions()
   }
   else
   {
-    delete mainwin->licqIcon;
+    if (mainwin->licqIcon)
+      delete mainwin->licqIcon;
     mainwin->licqIcon = NULL;
     mainwin->m_nDockMode = DockNone;
   }
@@ -541,12 +584,18 @@ void OptionsDlg::ApplyOptions()
   mainwin->licqDaemon->SetProxyAuthEnabled(chkProxyAuthEnabled->isChecked());
   mainwin->licqDaemon->SetProxyLogin(edtProxyLogin->text().local8Bit());
   mainwin->licqDaemon->SetProxyPasswd(edtProxyPasswd->text().local8Bit());
+  
+  mainwin->licqDaemon->setReconnectAfterUinClash(chkReconnectAfterUinClash->isChecked());
 
   mainwin->licqDaemon->SetIgnore(IGNORE_NEWUSERS, chkIgnoreNewUsers->isChecked());
   mainwin->licqDaemon->SetIgnore(IGNORE_MASSMSG, chkIgnoreMassMsg->isChecked());
   mainwin->licqDaemon->SetIgnore(IGNORE_WEBPANEL, chkIgnoreWebPanel->isChecked());
   mainwin->licqDaemon->SetIgnore(IGNORE_EMAILPAGER, chkIgnoreEmailPager->isChecked());
 
+  mainwin->licqDaemon->SetAutoUpdateInfo(chkAutoUpdateInfo->isChecked());
+  mainwin->licqDaemon->SetAutoUpdateInfoPlugins(chkAutoUpdateInfoPlugins->isChecked());
+  mainwin->licqDaemon->SetAutoUpdateStatusPlugins(chkAutoUpdateStatusPlugins->isChecked());
+
   // Plugin tab
   if (cmbUrlViewer->currentText() == DEFAULT_URL_VIEWER)
     mainwin->licqDaemon->setUrlViewer("none");  // untranslated!
@@ -609,14 +658,17 @@ void OptionsDlg::ApplyOptions()
   oem->SetParameters(txtSndPlayer.latin1(), oemparams);
   //TODO Make general for all plugins
   ICQOwner *o = gUserManager.FetchOwner(LOCK_W);
-  o->SetEnableSave(false);
-  o->SetAcceptInAway(chkOEAway->isChecked());
-  o->SetAcceptInNA(chkOENA->isChecked());
-  o->SetAcceptInOccupied(chkOEOccupied->isChecked());
-  o->SetAcceptInDND(chkOEDND->isChecked());
-  o->SetEnableSave(true);
-  o->SaveLicqInfo();
-  gUserManager.DropOwner();
+  if (o)
+  {
+    o->SetEnableSave(false);
+    o->SetAcceptInAway(chkOEAway->isChecked());
+    o->SetAcceptInNA(chkOENA->isChecked());
+    o->SetAcceptInOccupied(chkOEOccupied->isChecked());
+    o->SetAcceptInDND(chkOEDND->isChecked());
+    o->SetEnableSave(true);
+    o->SaveLicqInfo();
+    gUserManager.DropOwner();
+  }
   mainwin->licqDaemon->SetAlwaysOnlineNotify(chkAlwaysOnlineNotify->isChecked());
 }
 
@@ -761,8 +813,11 @@ QWidget* OptionsDlg::new_appearance_options()
   QWhatsThis::add(chkDockFortyEight, tr("Selects between the standard 64x64 icon used in the WindowMaker/Afterstep wharf "
                                         "and a shorter 64x48 icon for use in the Gnome/KDE panel."));
   rdbDockThemed = new QRadioButton(tr("Themed Icon"), boxDocking);
+  rdbDockSmall = new QRadioButton(tr("Small Icon"), boxDocking);
+  QWhatsThis::add(rdbDockSmall, tr("Uses the freedesktop.org standard to dock a small icon into the window manager.  Works with many different window managers."));
   grpDocking->insert(rdbDockDefault);
   grpDocking->insert(rdbDockThemed);
+  grpDocking->insert(rdbDockSmall);
   (void) new QLabel(tr("Theme:"), boxDocking);
   cmbDockTheme = new QComboBox(boxDocking);
   // Set the currently available themes
@@ -807,6 +862,7 @@ void OptionsDlg::slot_useDockToggled(bool b)
     cmbDockTheme->setEnabled(false);
     rdbDockDefault->setEnabled(false);
     rdbDockThemed->setEnabled(false);
+    rdbDockSmall->setEnabled(false);
     chkDockFortyEight->setEnabled(false);
 #endif
     chkHidden->setEnabled(false);
@@ -822,6 +878,7 @@ void OptionsDlg::slot_useDockToggled(bool b)
 #ifndef USE_KDE
   rdbDockDefault->setEnabled(true);
   rdbDockThemed->setEnabled(true);
+  rdbDockSmall->setEnabled(true);
   if (rdbDockDefault->isChecked())
   {
     cmbDockTheme->setEnabled(false);
@@ -849,6 +906,8 @@ QWidget* OptionsDlg::new_sounds_options()
   chkOnEvents = new QCheckBox(tr("OnEvents Enabled"), hor);
   QWidget* dummy = new QWidget(hor);
   dummy->setFixedSize(50, 1);
+  
+  connect(chkOnEvents, SIGNAL(toggled(bool)), this, SLOT(slot_chkOnEventsToggled(bool)));
 
   QWhatsThis::add(chkOnEvents, tr("Enable running of \"Command\" when the relevant "
                                  "event occurs."));
@@ -901,13 +960,34 @@ QWidget* OptionsDlg::new_sounds_options()
   QWhatsThis::add(chkAlwaysOnlineNotify, tr("Perform the online notify OnEvent "
      "when logging on (this is different from how the Mirabilis client works)"));
 
-  //lay->addStretch(1);
+  slot_chkOnEventsToggled(chkOnEvents->isChecked());
 
   return w;
 }
 
 // -----------------------------------------------------------------------------
+/*! 
+ * This slot enables/disables the relevant onEvent configuration widgets,
+ * depending on wether chkOnEvent is checked or not.
+ */
+void OptionsDlg::slot_chkOnEventsToggled(bool b_checked)
+{
+  edtSndPlayer->setEnabled(b_checked);
+  edtSndMsg->setEnabled(b_checked);
+  edtSndUrl->setEnabled(b_checked);
+  edtSndChat->setEnabled(b_checked);
+  edtSndFile->setEnabled(b_checked);
+  edtSndNotify->setEnabled(b_checked);
+  edtSndSysMsg->setEnabled(b_checked);
+  edtSndMsgSent->setEnabled(b_checked);
+  chkOEAway->setEnabled(b_checked);
+  chkOENA->setEnabled(b_checked);
+  chkOEOccupied->setEnabled(b_checked);
+  chkOEDND->setEnabled(b_checked);
+  chkAlwaysOnlineNotify->setEnabled(b_checked);
+}
 
+// -----------------------------------------------------------------------------
 QWidget *OptionsDlg::new_network_options()
 {
   QWidget *w = new QWidget(this);
@@ -930,17 +1010,18 @@ QWidget *OptionsDlg::new_network_options()
 
   chkFirewall = new QCheckBox(tr("I am behind a firewall"), gbFirewall);
   connect(chkFirewall, SIGNAL(toggled(bool)), SLOT(slot_useFirewall(bool)));
-  new QWidget(gbFirewall);
-  chkTCPEnabled = new QCheckBox(tr("I can receive direct connections"), gbFirewall);
-  connect(chkTCPEnabled, SIGNAL(toggled(bool)), SLOT(slot_usePortRange(bool)));
-  new QWidget(gbFirewall);
-  QLabel *lbl = new QLabel(tr("Port Range:"), gbFirewall);
+  QHBox *boxFw1 = new QHBox(gbFirewall);
+  QLabel *lbl = new QLabel(tr("Port Range:"), boxFw1);
   QWhatsThis::add(lbl, tr("TCP port range for incoming connections."));
-  spnPortLow = new QSpinBox(gbFirewall);
+  spnPortLow = new QSpinBox(boxFw1);
   spnPortLow->setRange(0, 0xFFFF);
   spnPortLow->setSpecialValueText(tr("Auto"));
-  lbl = new QLabel(tr("\tto"), gbFirewall);
-  spnPortHigh = new QSpinBox(gbFirewall);
+  
+  chkTCPEnabled = new QCheckBox(tr("I can receive direct connections"), gbFirewall);
+  connect(chkTCPEnabled, SIGNAL(toggled(bool)), SLOT(slot_usePortRange(bool)));
+  QHBox *boxFw2 = new QHBox(gbFirewall);
+  lbl = new QLabel(tr("\tto"), boxFw2);
+  spnPortHigh = new QSpinBox(boxFw2);
   spnPortHigh->setRange(0, 0xFFFF);
   spnPortHigh->setSpecialValueText(tr("Auto"));
 
@@ -974,6 +1055,16 @@ QWidget *OptionsDlg::new_network_options()
   connect(chkProxyEnabled, SIGNAL(toggled(bool)), SLOT(slot_useProxy(bool)));
   connect(chkProxyAuthEnabled, SIGNAL(toggled(bool)), edtProxyLogin, SLOT(setEnabled(bool)));
   connect(chkProxyAuthEnabled, SIGNAL(toggled(bool)), edtProxyPasswd, SLOT(setEnabled(bool)));
+  
+  QGroupBox *gbConnection = new QGroupBox(2, QGroupBox::Horizontal, w);
+  lay->addWidget(gbConnection);
+  gbConnection->setTitle(tr("Connection"));
+  
+  chkReconnectAfterUinClash = new QCheckBox(tr("Reconnect after Uin clash"), gbConnection);
+  QWhatsThis::add(chkReconnectAfterUinClash, tr("Licq can reconnect you when you got "
+                                                "disconnected because your Uin was used "
+                                                "from another location. Check this if you "
+                                                "want Licq to reconnect automatically."));
 
   lay->addStretch(1);
 
@@ -1356,6 +1447,22 @@ QWidget* OptionsDlg::new_misc_options()
 
   buildAutoStatusCombos(1);
 
+  boxAutoUpdate = new QGroupBox(2, Vertical, tr("Auto Update"), w);
+  lay->addWidget(boxAutoUpdate);
+  chkAutoUpdateInfo = new QCheckBox(tr("Auto Update Info"), boxAutoUpdate);
+  QWhatsThis::add(chkAutoUpdateInfo, tr("Automatically update users' server "
+                                        "stored information."));
+
+  chkAutoUpdateInfoPlugins = new QCheckBox(tr("Auto Update Info Plugins"),
+                                           boxAutoUpdate);
+  QWhatsThis::add(chkAutoUpdateInfoPlugins, tr("Automatically update users' "
+                                               "Phone Book and Picture."));
+
+  chkAutoUpdateStatusPlugins = new QCheckBox(tr("Auto Update Status Plugins"),
+                                             boxAutoUpdate);
+  QWhatsThis::add(chkAutoUpdateStatusPlugins, tr("Automatically update users' "
+                     "Phone \"Follow Me\", File Server and ICQphone status."));
+
   lay->addStretch(1);
   lay->activate();
 
