@@ -12,7 +12,6 @@
 #include <qwizard.h>
 #endif
 
-//TODO This entire class for protocol plugin support
 
 class QGroupBox;
 class QCheckBox;
@@ -36,8 +35,8 @@ signals:
 protected:
   CICQDaemon *server;
   QGroupBox *grpInfo;
-  CInfoField *nfoUin, *nfoPassword1, *nfoPassword2;
-  QCheckBox *chkExistingUser, *chkSavePassword;
+  CInfoField *nfoPassword1, *nfoPassword2;
+  QCheckBox *chkSavePassword;
   QWidget* page1, *page2, *page3;
   QLabel* lblInfo, *lblInfo2;
 
