@@ -30,12 +30,14 @@ signals:
   // Signal signals
   void signal_updatedList(CICQSignal *);
   void signal_updatedUser(CICQSignal *);
-  void signal_updatedStatus();
+  void signal_updatedStatus(CICQSignal *);
   void signal_logon();
   void signal_logoff();
   void signal_ui_viewevent(unsigned long);
-  void signal_ui_message(unsigned long);
+  void signal_ui_viewevent(const char *);
+  void signal_ui_message(const char *, unsigned long);
   void signal_protocolPlugin(unsigned long);
+  void signal_eventTag(const char *, unsigned long, unsigned long);
 
   // Event signals
   void signal_doneOwnerFcn(ICQEvent *);
