@@ -51,6 +51,7 @@ class CMainWindow;
 class CUserEvent;
 class CMMUserView;
 class CEButton;
+class CETabWidget;
 class CMessageViewWidget;
 class MLView;
 
@@ -69,13 +70,15 @@ public:
   bool tabIsSelected(QWidget *tab);
   bool tabExists(QWidget *tab);
   void updateTabLabel(ICQUser *u);
+  void gotTyping(ICQUser *u);
 
 private:
-  QTabWidget *tabw;
-
-public slots:
+  CETabWidget *tabw;
   void updateTitle(QWidget *tab);
   void clearEvents(QWidget *tab);
+
+public slots:
+  void slot_currentChanged(QWidget *tab);
   void moveLeft();
   void moveRight();
 
@@ -88,30 +91,30 @@ class UserEventCommon : public QWidget
 {
   Q_OBJECT
 public:
-
-  UserEventCommon(CICQDaemon *s, CSignalManager *theSigMan, CMainWindow *m,
-                  unsigned long _nUin, QWidget* parent = 0, const char* name =0);
-#ifdef QT_PROTOCOL_PLUGIN
   UserEventCommon(CICQDaemon *s, CSignalManager *theSigMan, CMainWindow *m,
                   const char *_szId, unsigned long _nPPID, QWidget *parent = 0,
                   const char *name = 0);
-#endif
   virtual ~UserEventCommon();
 
-  unsigned long Uin() { return m_nUin; }
-#ifdef QT_PROTOCOL_PLUGIN
   char *Id()  { return m_szId; }
   unsigned long PPID()  { return m_nPPID; }
-#endif
 
+  void AddEventTag(unsigned long n)  { if (n) m_lnEventTag.push_back(n); }
+  void gotTyping(unsigned short);
+ 
+  enum type {
+  	UC_MESSAGE,
+  	UC_URL,
+  	UC_CHAT,
+  	UC_FILE,
+  	UC_CONTACT,
+  	UC_SMS
+  };
 protected:
   QTextCodec *codec;
   bool m_bOwner;
-  unsigned long m_nUin;
-#ifdef QT_PROTOCOL_PLUGIN
   char *m_szId;
   unsigned long m_nPPID;
-#endif
   QBoxLayout* top_lay, *top_hlay;
   CICQDaemon *server;
   CMainWindow *mainwin;
@@ -123,7 +126,7 @@ protected:
 
   CInfoField *nfoStatus, *nfoTimezone;
   time_t m_nRemoteTimeOffset;
-  QTimer *tmrTime;
+  QTimer *tmrTime, *tmrTyping;
   bool m_bDeleteUser;
   QString m_sBaseTitle, m_sProgressMsg;
 
@@ -135,16 +138,18 @@ protected:
   void SetGeneralInfo(ICQUser *);
 
 protected slots:
+  void slot_connectsignal();
   void slot_userupdated(CICQSignal *);
   void slot_updatetime();
+  void slot_updatetyping();
   void showHistory();
   void showUserInfo();
-  void slot_usermenu() { gMainWindow->SetUserMenuUin(m_nUin); }
+  void slot_usermenu() { gMainWindow->SetUserMenuUser(m_szId, m_nPPID); }
   void slot_security();
   void slot_setEncoding(int encodingMib);
 
 signals:
-  void finished(unsigned long);
+  void finished(const char *, unsigned long);
   void encodingChanged();
   void viewurl(QWidget*, QString);
 };
@@ -157,12 +162,8 @@ class UserViewEvent : public UserEventCommon
   Q_OBJECT
 public:
 
-  UserViewEvent(CICQDaemon *s, CSignalManager *theSigMan, CMainWindow *m,
-                  unsigned long _nUin, QWidget* parent = 0);
-#ifdef QT_PROTOCOL_PLUGIN
   UserViewEvent(CICQDaemon *s, CSignalManager *theSigMan, CMainWindow *m,
                   const char *_szId, unsigned long _nPPID, QWidget *parent = 0);
-#endif
   virtual ~UserViewEvent();
 
 protected:
@@ -207,13 +208,9 @@ class UserSendCommon : public UserEventCommon
   Q_OBJECT
 public:
 
-  UserSendCommon(CICQDaemon *s, CSignalManager *theSigMan, CMainWindow *m,
-                 unsigned long _nUin, QWidget* parent = 0, const char* name=0);
-#ifdef QT_PROTOCOL_PLUGIN
   UserSendCommon(CICQDaemon *s, CSignalManager *theSigMan, CMainWindow *m,
                  const char *_szId, unsigned long _nPPID, QWidget *parent = 0,
                  const char *name = 0);
-#endif
   virtual ~UserSendCommon();
 
   void setText(const QString& txt);
@@ -238,6 +235,8 @@ protected:
   CMMUserView *lstMultipleRecipients;
   MLEditWrap *mleSend;
   CICQColor icqColor;
+  QString strTempMsg;
+  QTimer *tmrSendTyping;
 
   void RetrySend(ICQEvent *e, bool bOnline, unsigned short nLevel);
   virtual void UserUpdated(CICQSignal *, ICQUser *);
@@ -258,6 +257,8 @@ protected slots:
   void slot_SetBackgroundICQColor();
   void trySecure();
   void slot_ClearNewEvents();
+  void slot_textChanged();
+  void slot_textChanged_timeout();
 
 public slots:
   void changeEventType(int);
@@ -279,16 +280,14 @@ public:
 
   UserSendMsgEvent(CICQDaemon *s, CSignalManager *theSigMan, CMainWindow *m,
                   unsigned long _nUin, QWidget* parent = 0);
-#ifdef QT_PROTOCOL_PLUGIN
   UserSendMsgEvent(CICQDaemon *s, CSignalManager *theSigMan, CMainWindow *m,
                   const char *_szId, unsigned long _nPPID, QWidget *parent = 0);
-#endif
   virtual ~UserSendMsgEvent();
 
 protected:
   virtual bool sendDone(ICQEvent *);
   virtual void resetSettings();
-  virtual bool isType(int id) { return (id == 0); }
+  virtual bool isType(int id) { return (id == UC_MESSAGE); }
 
 protected slots:
   virtual void sendButton();
@@ -302,12 +301,8 @@ class UserSendUrlEvent : public UserSendCommon
   Q_OBJECT
 public:
 
-  UserSendUrlEvent(CICQDaemon *s, CSignalManager *theSigMan, CMainWindow *m,
-                  unsigned long _nUin, QWidget* parent = 0);
-#ifdef QT_PROTOCOL_PLUGIN
   UserSendUrlEvent(CICQDaemon *s, CSignalManager *theSigMan, CMainWindow *m,
                   const char *_szId, unsigned long _nPPID, QWidget *parent = 0);
-#endif
   virtual ~UserSendUrlEvent();
 
   void setUrl(const QString& url, const QString& description);
@@ -317,7 +312,7 @@ protected:
   CInfoField *edtItem;
   virtual bool sendDone(ICQEvent *);
   virtual void resetSettings();
-  virtual bool isType(int id) { return (id == 1); }
+  virtual bool isType(int id) { return (id == UC_URL); }
 
 protected slots:
   virtual void sendButton();
@@ -331,12 +326,8 @@ class UserSendFileEvent : public UserSendCommon
   Q_OBJECT
 public:
 
-  UserSendFileEvent(CICQDaemon *s, CSignalManager *theSigMan, CMainWindow *m,
-                  unsigned long _nUin, QWidget* parent = 0);
-#ifdef QT_PROTOCOL_PLUGIN
   UserSendFileEvent(CICQDaemon *s, CSignalManager *theSigMan, CMainWindow *m,
                   const char *_szId, unsigned long _nPPID, QWidget *parent = 0);
-#endif
   virtual ~UserSendFileEvent();
 
   void setFile(const QString& file, const QString& description);
@@ -348,7 +339,7 @@ protected:
   ConstFileList m_lFileList;
   virtual bool sendDone(ICQEvent*);
   virtual void resetSettings();
-  virtual bool isType(int id) { return (id == 2); }
+  virtual bool isType(int id) { return (id == UC_FILE); }
 
 protected slots:
   void browseFile();
@@ -365,12 +356,8 @@ class UserSendChatEvent : public UserSendCommon
   Q_OBJECT
 public:
 
-  UserSendChatEvent(CICQDaemon *s, CSignalManager *theSigMan, CMainWindow *m,
-                    unsigned long _nUin, QWidget* parent = 0);
-#ifdef QT_PROTOCOL_PLUGIN
   UserSendChatEvent(CICQDaemon *s, CSignalManager *theSigMan, CMainWindow *m,
                   const char *_szId, unsigned long _nPPID, QWidget *parent = 0);
-#endif
   virtual ~UserSendChatEvent();
 
 protected:
@@ -381,7 +368,7 @@ protected:
   unsigned short m_nMPChatPort;
   virtual bool sendDone(ICQEvent *);
   virtual void resetSettings();
-  virtual bool isType(int id) { return (id == 3); }
+  virtual bool isType(int id) { return (id == UC_CHAT); }
 
 protected slots:
   virtual void sendButton();
@@ -399,21 +386,18 @@ public:
 
   UserSendContactEvent(CICQDaemon *s, CSignalManager *theSigMan, CMainWindow *m,
                        unsigned long _nUin, QWidget* parent = 0);
-#ifdef QT_PROTOCOL_PLUGIN
   UserSendContactEvent(CICQDaemon *s, CSignalManager *theSigMan, CMainWindow *m,
                   const char *_szId, unsigned long _nPPID, QWidget *parent = 0);
-#endif
   virtual ~UserSendContactEvent();
 
-  //TODO
-  void setContact(unsigned long uin, const QString& alias);
+  void setContact(const char *, unsigned long, const QString& alias);
 
 protected:
   CMMUserView *lstContacts;
 
   virtual bool sendDone(ICQEvent *);
   virtual void resetSettings();
-  virtual bool isType(int id) { return (id == 4); }
+  virtual bool isType(int id) { return (id == UC_CONTACT); }
 
 protected slots:
   virtual void sendButton();
@@ -427,12 +411,8 @@ class UserSendSmsEvent : public UserSendCommon
   Q_OBJECT
 public:
 
-  UserSendSmsEvent(CICQDaemon *s, CSignalManager *theSigMan, CMainWindow *m,
-                  unsigned long _nUin, QWidget* parent = 0);
-#ifdef QT_PROTOCOL_PLUGIN
   UserSendSmsEvent(CICQDaemon *s, CSignalManager *theSigMan, CMainWindow *m,
                   const char *_szId, unsigned long _nPPID, QWidget *parent = 0);
-#endif
   virtual ~UserSendSmsEvent();
 
 protected:
@@ -443,11 +423,11 @@ protected:
 
   virtual bool sendDone(ICQEvent *);
   virtual void resetSettings();
-  virtual bool isType(int id) { return (id == 5); }
+  virtual bool isType(int id) { return (id == UC_SMS); }
 
 protected slots:
   virtual void sendButton();
-  
+
   void slot_count();
 };
 
