@@ -20,6 +20,9 @@
 #include "licq_sighandler.h"
 #include "support.h"
 
+// Localization
+#include "gettext.h"
+
 #define DEBUG_THREADS(x)
 
 
@@ -45,6 +48,10 @@ CPChat_Color::CPChat_Color(const char *_sLocalName, unsigned short _nLocalPort,
   m_szName = NULL;
   m_nPort = _nLocalPort;
   m_nUin = gUserManager.OwnerUin();
+  char szUin[24];
+  sprintf(szUin, "%lu", m_nUin);
+  m_szId = strdup(szUin);
+  m_nPPID = LICQ_PPID;
   m_nColorForeRed = nColorForeRed;
   m_nColorForeGreen = nColorForeGreen;
   m_nColorForeBlue = nColorForeBlue;
@@ -79,6 +86,10 @@ CPChat_Color::CPChat_Color(CBuffer &b)
   b.UnpackUnsignedLong();
   b.UnpackUnsignedLong();
   m_nUin = b.UnpackUnsignedLong();
+  char szUin[24];
+  sprintf(szUin, "%lu", m_nUin);
+  m_szId = strdup(szUin);
+  m_nPPID = LICQ_PPID;
   m_szName = strdup(b.UnpackString(buf, sizeof(buf)));
   m_nPort = b.UnpackUnsignedShort();
   m_nPort = (m_nPort >> 8) + (m_nPort << 8);
@@ -96,8 +107,9 @@ CPChat_Color::CPChat_Color(CBuffer &b)
 //-----ChatColorFont----------------------------------------------------------------
 CChatClient::CChatClient()
 {
-  m_nVersion = m_nUin = m_nIp = m_nIntIp = m_nPort = m_nMode
+  m_nVersion = m_nUin = m_nPPID = m_nIp = m_nIntIp = m_nPort = m_nMode
      = m_nSession = m_nHandshake = 0;
+  m_szId = NULL;
 }
 
 
@@ -105,6 +117,8 @@ CChatClient::CChatClient(ICQUser *u)
 {
   m_nVersion = u->Version();
   m_nUin = u->Uin();
+  m_szId = strdup(u->IdString());
+  m_nPPID = u->PPID();
   m_nIp = u->Ip();
   m_nIntIp = u->IntIp();
   m_nMode = u->Mode();
@@ -122,12 +136,42 @@ CChatClient::CChatClient(CBuffer &b)
   LoadFromBuffer(b);
 }
 
+CChatClient::CChatClient(const CChatClient &p)
+{
+  m_szId = NULL;
+  *this = p;
+}
+
+CChatClient& CChatClient::operator=(const CChatClient &p)
+{
+  if (this != &p)
+  {
+    m_nVersion = p.m_nVersion;
+    m_nPort = p.m_nPort;
+    m_nUin = p.m_nUin;
+    m_nPPID = p.m_nPPID;
+    m_nIp = p.m_nIp;
+    m_nIntIp = p.m_nIntIp;
+    m_nMode = p.m_nMode;
+    m_nSession = p.m_nSession;
+    m_nHandshake = p.m_nHandshake;
+    if (m_szId)
+      free(m_szId);
+    m_szId = p.m_szId ? strdup(p.m_szId) : NULL;
+  }
+  return *this;
+}
+
 bool CChatClient::LoadFromBuffer(CBuffer &b)
 {
   m_nVersion = b.UnpackUnsignedLong();
   m_nPort = b.UnpackUnsignedShort();
   b.UnpackUnsignedShort();
   m_nUin = b.UnpackUnsignedLong();
+  char szUin[24];
+  sprintf(szUin, "%lu", m_nUin);
+  m_szId = strdup(szUin);
+  m_nPPID = LICQ_PPID;
   m_nIp = b.UnpackUnsignedLong();
   m_nIntIp = b.UnpackUnsignedLong();
   m_nMode = b.UnpackChar();
@@ -148,6 +192,10 @@ bool CChatClient::LoadFromHandshake_v2(CBuffer &b)
   m_nVersion = b.UnpackUnsignedLong();
   b.UnpackUnsignedLong();
   m_nUin = b.UnpackUnsignedLong();
+  char szUin[24];
+  sprintf(szUin, "%lu", m_nUin);
+  m_szId = strdup(szUin);
+  m_nPPID = LICQ_PPID;
   m_nIp = b.UnpackUnsignedLong();
   m_nIntIp = b.UnpackUnsignedLong();
   m_nMode = b.UnpackChar();
@@ -170,6 +218,10 @@ bool CChatClient::LoadFromHandshake_v4(CBuffer &b)
   m_nVersion = b.UnpackUnsignedLong();
   b.UnpackUnsignedLong();
   m_nUin = b.UnpackUnsignedLong();
+  char szUin[24];
+  sprintf(szUin, "%lu", m_nUin);
+  m_szId = strdup(szUin);
+  m_nPPID = LICQ_PPID;
   m_nIp = b.UnpackUnsignedLong();  // Will probably be zero...
   m_nIntIp = b.UnpackUnsignedLong();
   m_nMode = b.UnpackChar();
@@ -189,6 +241,10 @@ bool CChatClient::LoadFromHandshake_v6(CBuffer &b)
 
   m_nVersion = hand.VersionMajor();
   m_nUin = hand.SourceUin();
+  char szUin[24];
+  sprintf(szUin, "%lu", m_nUin);
+  m_szId = strdup(szUin);
+  m_nPPID = LICQ_PPID;
   m_nIntIp = hand.LocalIp();
   m_nIp = hand.RealIp();
   m_nMode = hand.Mode();
@@ -208,6 +264,10 @@ bool CChatClient::LoadFromHandshake_v7(CBuffer &b)
 
   m_nVersion = hand.VersionMajor();
   m_nUin = hand.SourceUin();
+  char szUin[24];
+  sprintf(szUin, "%lu", m_nUin);
+  m_szId = strdup(szUin);
+  m_nPPID = LICQ_PPID;
   m_nIntIp = hand.LocalIp();
   m_nIp = hand.RealIp();
   m_nMode = hand.Mode();
@@ -233,6 +293,10 @@ CPChat_ColorFont::CPChat_ColorFont(const char *szLocalName, unsigned short nLoca
   m_szName = NULL;
   m_nPort = nLocalPort;
   m_nUin = gUserManager.OwnerUin();
+  char szUin[24];
+  sprintf(szUin, "%lu", m_nUin);
+  m_szId = strdup(szUin);
+  m_nPPID = LICQ_PPID;
   m_nColorForeRed = nColorForeRed;
   m_nColorForeGreen = nColorForeGreen;
   m_nColorForeBlue = nColorForeBlue;
@@ -300,6 +364,10 @@ CPChat_ColorFont::CPChat_ColorFont(CBuffer &b)
 
   b.UnpackUnsignedLong();
   m_nUin = b.UnpackUnsignedLong();
+  char szUin[24];
+  sprintf(szUin, "%lu", m_nUin);
+  m_szId = strdup(szUin);
+  m_nPPID = LICQ_PPID;
   m_szName = strdup(b.UnpackString(buf, sizeof(buf)));
   m_nColorForeRed = (unsigned char)b.UnpackChar();
   m_nColorForeGreen = (unsigned char)b.UnpackChar();
@@ -530,6 +598,8 @@ CPChat_Beep::CPChat_Beep()
 CChatUser::CChatUser()
 {
   uin = 0;
+  szId = 0;
+  nPPID = 0;
   nToKick = 0;
   state = CHAT_STATE_DISCONNECTED;
   colorFore[0] = colorFore[1] = colorFore[2] = 0x00;
@@ -567,6 +637,9 @@ CChatEvent::~CChatEvent()
 
 //=====ChatManager===========================================================
 ChatManagerList CChatManager::cmList;
+pthread_mutex_t CChatManager::cmList_mutex = PTHREAD_MUTEX_INITIALIZER;
+pthread_mutex_t CChatManager::waiting_thread_cancel_mutex
+                                                  = PTHREAD_MUTEX_INITIALIZER;
 
 
 CChatManager::CChatManager(CICQDaemon *d, unsigned long nUin,
@@ -579,6 +652,11 @@ CChatManager::CChatManager(CICQDaemon *d, unsigned long nUin,
   pipe(pipe_events);
 
   m_nUin = nUin;
+  char szUin[24];
+  sprintf(szUin, "%lu", m_nUin);
+  m_szId = strdup(szUin);
+  m_nPPID = LICQ_PPID;
+
 //  m_nSession = rand();
   licqDaemon = d;
 
@@ -607,8 +685,13 @@ CChatManager::CChatManager(CICQDaemon *d, unsigned long nUin,
   m_bFocus = true;
   m_bSleep = false;
   m_pChatClient = NULL;
+  m_bThreadCreated = false;
 
+  pthread_mutex_init(&thread_list_mutex, NULL);
+
+  pthread_mutex_lock(&cmList_mutex);
   cmList.push_back(this);
+  pthread_mutex_unlock(&cmList_mutex);
 }
 
 
@@ -617,7 +700,7 @@ bool CChatManager::StartChatServer()
 {
   if (licqDaemon->StartTCPServer(&chatServer) == -1)
   {
-    gLog.Warn("%sNo more ports available, add more or close open chat/file sessions.\n", L_WARNxSTR);
+    gLog.Warn(tr("%sNo more ports available, add more or close open chat/file sessions.\n"), L_WARNxSTR);
     return false;
   }
 
@@ -645,6 +728,8 @@ bool CChatManager::StartAsServer()
     return false;
   }
 
+  m_bThreadCreated = true;
+
   return true;
 }
 
@@ -654,7 +739,7 @@ void CChatManager::StartAsClient(unsigned short nPort)
 {
   if (!StartChatServer()) return;
 
-  ICQUser *u = gUserManager.FetchUser(m_nUin, LOCK_R);
+  ICQUser *u = gUserManager.FetchUser(m_szId, LICQ_PPID, LOCK_R);
   if (u == NULL) return;
   m_pChatClient = new CChatClient(u);
   m_pChatClient->m_nPort = nPort;
@@ -676,26 +761,71 @@ bool CChatManager::ConnectToChat(CChatClient *c)
   u->m_pClient = c;
   u->m_pClient->m_nSession = m_nSession;
   u->uin = c->m_nUin;
+  u->szId = strdup(c->m_szId);
+  u->nPPID = c->m_nPPID;
 
   bool bSendIntIp = false;
+  bool bTryDirect = true;
+  bool bResult = false;
   ICQUser *temp_user = gUserManager.FetchUser(u->uin, LOCK_R);
   if (temp_user != NULL)
   {
     bSendIntIp = temp_user->SendIntIp();
+    bTryDirect = temp_user->Version() <= 6 || temp_user->Mode() == MODE_DIRECT;
     gUserManager.DropUser(temp_user);
   }
 
-  gLog.Info("%sChat: Connecting to server.\n", L_TCPxSTR);
-  if (!licqDaemon->OpenConnectionToUser("chat", c->m_nIp, c->m_nIntIp, &u->sock,
-          c->m_nPort, bSendIntIp))
+  bool bSuccess = false;
+  if (bTryDirect)
   {
-    delete u;
-    return false;
+    gLog.Info("%sChat: Connecting to server.\n", L_TCPxSTR);
+    bSuccess = licqDaemon->OpenConnectionToUser("chat", c->m_nIp, c->m_nIntIp,
+                                            &u->sock, c->m_nPort, bSendIntIp);
   }
 
-  chatUsers.push_back(u);
+  if (!bSuccess)
+  {
+    ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+    unsigned long nIp = bSendIntIp ? o->IntIp() : o->Ip();
+    gUserManager.DropOwner();
+  
+    // try reverse connect
+    int nId = licqDaemon->RequestReverseConnection(c->m_nUin, c->m_nSession,
+                                                 nIp, LocalPort(), c->m_nPort);
+    if (nId != -1)
+    {
+      pthread_t t;
+      struct SChatReverseConnectInfo *r = new struct SChatReverseConnectInfo;
+      r->nId = nId;
+      r->u = u;
+      r->m = this;
+      r->bTryDirect = !bTryDirect;
+      pthread_mutex_lock(&thread_list_mutex);
+      pthread_create(&t, NULL, ChatWaitForSignal_tep, r);
+      waitingThreads.push_back(t);
+      pthread_mutex_unlock(&thread_list_mutex);
+      bResult = true;
+    }
+    else
+    {
+      delete u->m_pClient;
+      delete u;
+    }
+  }
+  else
+  {
+    chatUsers.push_back(u);
+    bResult = SendChatHandshake(u);
+  }
+ 
+  return bResult;
+}
 
-  gLog.Info("%sChat: Shaking hands [v%d].\n", L_TCPxSTR, VersionToUse(c->m_nVersion));
+bool CChatManager::SendChatHandshake(CChatUser *u)
+{
+  CChatClient *c = u->m_pClient;
+  
+  gLog.Info(tr("%sChat: Shaking hands [v%d].\n"), L_TCPxSTR, VersionToUse(c->m_nVersion));
 
   // Send handshake packet:
   if (!CICQDaemon::Handshake_Send(&u->sock, c->m_nUin, LocalPort(),
@@ -708,7 +838,7 @@ bool CChatManager::ConnectToChat(CChatClient *c)
      m_nColorBack[0], m_nColorBack[1], m_nColorBack[2]);
   u->sock.SendPacket(p_color.getBuffer());
 
-  gLog.Info("%sChat: Waiting for color/font response.\n", L_TCPxSTR);
+  gLog.Info(tr("%sChat: Waiting for color/font response.\n"), L_TCPxSTR);
 
   u->state = CHAT_STATE_WAITxFORxCOLORxFONT;
 
@@ -725,8 +855,11 @@ void CChatManager::AcceptReverseConnection(TCPSocket *s)
   CChatUser *u = new CChatUser;
   u->sock.TransferConnectionFrom(*s);
 
+  u->m_pClient = new CChatClient();
   u->m_pClient->m_nVersion = s->Version();
   u->m_pClient->m_nUin = s->Owner();
+  u->m_pClient->m_szId = s->OwnerId();
+  u->m_pClient->m_nPPID = s->OwnerPPID();
   u->m_pClient->m_nIp = s->RemoteIp();
   u->m_pClient->m_nIntIp = s->RemoteIp();
   u->m_pClient->m_nMode = MODE_DIRECT;
@@ -737,6 +870,8 @@ void CChatManager::AcceptReverseConnection(TCPSocket *s)
   u->m_pClient->m_nSession = 0;
 
   u->uin = u->m_pClient->m_nUin;
+  u->szId = strdup(u->m_pClient->m_szId);
+  u->nPPID = u->m_pClient->m_nPPID;
   u->state = CHAT_STATE_WAITxFORxCOLOR;
   chatUsers.push_back(u);
 
@@ -745,7 +880,7 @@ void CChatManager::AcceptReverseConnection(TCPSocket *s)
   sockman.DropSocket(&u->sock);
   write(pipe_thread[PIPE_WRITE], "R", 1);
 
-  gLog.Info("%sChat: Received reverse connection.\n", L_TCPxSTR);
+  gLog.Info(tr("%sChat: Received reverse connection.\n"), L_TCPxSTR);
 }
 
 
@@ -771,9 +906,9 @@ bool CChatManager::ProcessPacket(CChatUser *u)
   {
     char buf[128];
     if (u->sock.Error() == 0)
-      gLog.Info("%sChat: Remote end disconnected.\n", L_TCPxSTR);
+      gLog.Info(tr("%sChat: Remote end disconnected.\n"), L_TCPxSTR);
     else
-      gLog.Info("%sChat: Lost remote end:\n%s%s\n", L_TCPxSTR,
+      gLog.Info(tr("%sChat: Lost remote end:\n%s%s\n"), L_TCPxSTR,
                 L_BLANKxSTR, u->sock.ErrorStr(buf, 128));
     return false;
   }
@@ -786,7 +921,7 @@ bool CChatManager::ProcessPacket(CChatUser *u)
     {
       CBuffer handshake = u->sock.RecvBuffer();
       // get the handshake packet
-      if (!CICQDaemon::Handshake_Recv(&u->sock, LocalPort(), false))
+      if (!CICQDaemon::Handshake_Recv(&u->sock, LocalPort(), false, true))
       {
         gLog.Warn("%sChat: Bad handshake.\n", L_ERRORxSTR);
         return false;
@@ -810,16 +945,57 @@ bool CChatManager::ProcessPacket(CChatUser *u)
           u->m_pClient->LoadFromHandshake_v7(handshake);
           break;
       }
-      gLog.Info("%sChat: Received handshake from %ld [v%ld].\n", L_TCPxSTR,
-         u->m_pClient->m_nUin, u->sock.Version());
+      gLog.Info(tr("%sChat: Received handshake from %s [v%ld].\n"), L_TCPxSTR,
+         u->m_pClient->m_szId, u->sock.Version());
       u->uin = u->m_pClient->m_nUin;
-      u->state = CHAT_STATE_WAITxFORxCOLOR;
+      if (u->szId)  free(u->szId);
+      u->szId = strdup(u->m_pClient->m_szId);
+      u->nPPID = u->m_pClient->m_nPPID;
+      
+      bool bFound = false;
+      if (u->m_pClient->m_nId != 0)
+      {
+        pthread_mutex_lock(&licqDaemon->mutex_reverseconnect);
+        std::list<CReverseConnectToUserData *>::iterator iter;
+        for (iter = licqDaemon->m_lReverseConnect.begin();
+                         iter != licqDaemon->m_lReverseConnect.end();  iter++)
+        {
+          if ((*iter)->nId == u->m_pClient->m_nId && (*iter)->nUin == u->uin)
+          {
+            bFound = true;
+            (*iter)->bSuccess = true;
+            (*iter)->bFinished = true;
+            u->m_pClient->m_nSession = (*iter)->nData;
+            u->m_pClient->m_nPort = (*iter)->nPort;
+            pthread_cond_broadcast(&licqDaemon->cond_reverseconnect_done);
+            break;
+          }
+        }
+        pthread_mutex_unlock(&licqDaemon->mutex_reverseconnect);
+      }
+
+      if (bFound)
+      {
+        // Send color packet
+        CPChat_Color p_color(m_szName, LocalPort(),
+        m_nColorFore[0], m_nColorFore[1], m_nColorFore[2],
+        m_nColorBack[0], m_nColorBack[1], m_nColorBack[2]);
+        u->sock.SendPacket(p_color.getBuffer());
+
+        gLog.Info("%sChat: Waiting for color/font response.\n", L_TCPxSTR);
+
+        u->state = CHAT_STATE_WAITxFORxCOLORxFONT;
+      }
+      else
+      {
+        u->state = CHAT_STATE_WAITxFORxCOLOR;
+      }
       break;
     }
 
     case CHAT_STATE_WAITxFORxCOLOR:  // we just received the color packet
     {
-      gLog.Info("%sChat: Received color packet.\n", L_TCPxSTR);
+      gLog.Info(tr("%sChat: Received color packet.\n"), L_TCPxSTR);
 
       CPChat_Color pin(u->sock.RecvBuffer());
 
@@ -843,7 +1019,8 @@ bool CChatManager::ProcessPacket(CChatUser *u)
       for (iter = chatUsers.begin(); iter != chatUsers.end(); iter++)
       {
         // Skip this guys client info and anybody we haven't connected to yet
-        if ((*iter)->uin == u->uin || (*iter)->m_pClient->m_nUin == 0) continue;
+        if ((strcmp((*iter)->szId, u->szId) == 0 && (*iter)->nPPID == u->nPPID)
+            || (*iter)->m_pClient->m_szId == 0) continue;
         l.push_back((*iter)->m_pClient);
       }
 
@@ -866,7 +1043,7 @@ bool CChatManager::ProcessPacket(CChatUser *u)
 
     case CHAT_STATE_WAITxFORxFONT:
     {
-      gLog.Info("%sChat: Received font packet.\n", L_TCPxSTR);
+      gLog.Info(tr("%sChat: Received font packet.\n"), L_TCPxSTR);
       CPChat_Font pin(u->sock.RecvBuffer());
 
       // just received the font reply
@@ -885,10 +1062,12 @@ bool CChatManager::ProcessPacket(CChatUser *u)
 
     case CHAT_STATE_WAITxFORxCOLORxFONT:
     {
-      gLog.Info("%sChat: Received color/font packet.\n", L_TCPxSTR);
+      gLog.Info(tr("%sChat: Received color/font packet.\n"), L_TCPxSTR);
 
       CPChat_ColorFont pin(u->sock.RecvBuffer());
       u->uin = pin.Uin();
+      u->szId = strdup(pin.Id());
+      u->nPPID = pin.PPID();
 //      m_nSession = pin.Session();
 
       // just received the color/font packet
@@ -915,7 +1094,7 @@ bool CChatManager::ProcessPacket(CChatUser *u)
       // Parse the multiusers list
       if (pin.ChatClients().size() > 0)
       {
-        gLog.Info("%sChat: Joined multiparty (%d people).\n", L_TCPxSTR,
+        gLog.Info(tr("%sChat: Joined multiparty (%d people).\n"), L_TCPxSTR,
            pin.ChatClients().size() + 1);
         ChatClientList::iterator iter;
         for (iter = pin.ChatClients().begin(); iter != pin.ChatClients().end(); iter++)
@@ -923,7 +1102,8 @@ bool CChatManager::ProcessPacket(CChatUser *u)
           ChatUserList::iterator iter2;
           for (iter2 = chatUsers.begin(); iter2 != chatUsers.end(); iter2++)
           {
-            if ((*iter2)->uin == iter->m_nUin) break;
+            if (strcmp((*iter2)->szId, iter->m_szId) == 0 &&
+                (*iter2)->nPPID == iter->m_nPPID) break;
           }
           if (iter2 != chatUsers.end()) continue;
           // Connect to this user
@@ -999,9 +1179,9 @@ bool CChatManager::ProcessRaw(CChatUser *u)
   {
     char buf[128];
     if (u->sock.Error() == 0)
-      gLog.Info("%sChat: Remote end disconnected.\n", L_TCPxSTR);
+      gLog.Info(tr("%sChat: Remote end disconnected.\n"), L_TCPxSTR);
     else
-      gLog.Info("%sChat: Lost remote end:\n%s%s\n", L_TCPxSTR,
+      gLog.Info(tr("%sChat: Lost remote end:\n%s%s\n"), L_TCPxSTR,
                 L_BLANKxSTR, u->sock.ErrorStr(buf, 128));
     return false;
   }
@@ -1736,7 +1916,7 @@ bool CChatManager::SendBufferToClient(CBuffer *b, unsigned char cmd, CChatUser *
   if (!u->sock.SendRaw(&b_out))
   {
     char buf[128];
-    gLog.Warn("%sChat: Send error:\n%s%s\n", L_WARNxSTR, L_BLANKxSTR,
+    gLog.Warn(tr("%sChat: Send error:\n%s%s\n"), L_WARNxSTR, L_BLANKxSTR,
                u->sock.ErrorStr(buf, 128));
     CloseClient(u);
     return false;
@@ -1767,7 +1947,7 @@ void CChatManager::SendBuffer_Raw(CBuffer *b)
       if (!u->sock.SendRaw(b))
       {
         char buf[128];
-        gLog.Warn("%sChat: Send error:\n%s%s\n", L_WARNxSTR, L_BLANKxSTR,
+        gLog.Warn(tr("%sChat: Send error:\n%s%s\n"), L_WARNxSTR, L_BLANKxSTR,
                    u->sock.ErrorStr(buf, 128));
         CloseClient(u);
         ok = false;
@@ -2011,7 +2191,9 @@ void CChatManager::CloseChat()
   if (pipe_thread[PIPE_WRITE] != -1)
   {
     write(pipe_thread[PIPE_WRITE], "X", 1);
-    pthread_join(thread_chat, NULL);
+    if (m_bThreadCreated)
+      pthread_join(thread_chat, NULL);
+    m_bThreadCreated = false;
 
     close(pipe_thread[PIPE_READ]);
     close(pipe_thread[PIPE_WRITE]);
@@ -2136,7 +2318,6 @@ void *ChatManager_tep(void *arg)
     if (!chatman->ConnectToChat(chatman->m_pChatClient))
     {
       chatman->PushChatEvent(new CChatEvent(CHAT_ERRORxCONNECT, NULL));
-      delete chatman->m_pChatClient;
       return NULL;
     }
     chatman->m_pChatClient = 0;
@@ -2186,7 +2367,7 @@ void *ChatManager_tep(void *arg)
 
           u->state = CHAT_STATE_HANDSHAKE;
           chatman->chatUsers.push_back(u);
-          gLog.Info("%sChat: Received connection.\n", L_TCPxSTR);
+          gLog.Info(tr("%sChat: Received connection.\n"), L_TCPxSTR);
         }
 
         // Message from connected socket----------------------------------------
@@ -2195,7 +2376,7 @@ void *ChatManager_tep(void *arg)
           CChatUser *u = chatman->FindChatUser(nCurrentSocket);
           if (u == NULL)
           {
-            gLog.Warn("%sChat: No user owns socket %d.\n", L_WARNxSTR, nCurrentSocket);
+            gLog.Warn(tr("%sChat: No user owns socket %d.\n"), L_WARNxSTR, nCurrentSocket);
           }
           else
           {
@@ -2227,20 +2408,186 @@ void *ChatManager_tep(void *arg)
   return NULL;
 }
 
+void *ChatWaitForSignal_tep(void *arg)
+{
+  pthread_detach(pthread_self());
+
+  CICQDaemon *d;
+  struct SChatReverseConnectInfo *rc = (struct SChatReverseConnectInfo *)arg;
+  pthread_mutex_t *cancel_mutex = &CChatManager::waiting_thread_cancel_mutex;
+
+  pthread_mutex_lock(cancel_mutex);
+  pthread_cleanup_push(ChatWaitForSignal_cleanup, arg);
+    pthread_testcancel();
+  pthread_cleanup_pop(0);
+  d = rc->m->licqDaemon;
+  pthread_mutex_unlock(cancel_mutex);
+
+  pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
+  gLog.Info("%sChat: Waiting for reverse connection.\n", L_TCPxSTR);
+  bool bConnected = d->WaitForReverseConnection(rc->nId, rc->u->Uin());
+  pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
+
+  pthread_mutex_lock(cancel_mutex);
+  pthread_cleanup_push(ChatWaitForSignal_cleanup, arg);
+    pthread_testcancel();
+  pthread_cleanup_pop(0);
+
+  if (bConnected || !rc->bTryDirect)
+  {
+    if (!bConnected && rc->m->chatUsers.empty())
+      rc->m->PushChatEvent(new CChatEvent(CHAT_ERRORxCONNECT, NULL));
+
+
+    pthread_mutex_lock(&rc->m->thread_list_mutex);
+    ThreadList::iterator iter;
+    for (iter = rc->m->waitingThreads.begin();
+                                 iter != rc->m->waitingThreads.end(); iter++)
+    {
+      if (pthread_equal(*iter, pthread_self()))
+      {
+        rc->m->waitingThreads.erase(iter);
+        break;
+      }
+    }
+    pthread_mutex_unlock(&rc->m->thread_list_mutex);
+    pthread_mutex_unlock(cancel_mutex);
+
+    delete rc->u->m_pClient;
+    delete rc->u;
+    delete rc;
+    pthread_exit(NULL);
+  }
+
+  pthread_mutex_unlock(cancel_mutex);
+
+  bool bSendIntIp = false;
+  ICQUser *temp_user = gUserManager.FetchUser(rc->u->Uin(), LOCK_R);
+  if (temp_user != NULL)
+  {
+    bSendIntIp = temp_user->SendIntIp();
+    gUserManager.DropUser(temp_user);
+
+    pthread_mutex_lock(cancel_mutex);
+    pthread_cleanup_push(ChatWaitForSignal_cleanup, arg);
+      pthread_testcancel();
+    pthread_cleanup_pop(0);
+
+    unsigned long nIp = rc->u->m_pClient->m_nIp;
+    unsigned long nIntIp = rc->u->m_pClient->m_nIntIp;
+    unsigned short nPort = rc->u->m_pClient->m_nPort;
+    pthread_mutex_unlock(cancel_mutex);
+
+    pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
+    gLog.Info("%sChat: Reverse connection failed, trying direct.\n", L_TCPxSTR);
+    bool bSuccess = d->OpenConnectionToUser("chat", nIp, nIntIp, &rc->u->sock,
+                                            nPort, bSendIntIp);
+    pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
+    if (bSuccess)
+    {
+      pthread_mutex_lock(cancel_mutex);
+      pthread_cleanup_push(ChatWaitForSignal_cleanup, arg);
+        pthread_testcancel();
+      pthread_cleanup_pop(0);
+
+      if (rc->m->SendChatHandshake(rc->u))
+      {
+        rc->m->chatUsers.push_back(rc->u);
+
+        pthread_mutex_lock(&rc->m->thread_list_mutex);
+        ThreadList::iterator iter;
+        for (iter = rc->m->waitingThreads.begin();
+                                   iter != rc->m->waitingThreads.end(); iter++)
+        {
+          if (pthread_equal(*iter, pthread_self()))
+          {
+            rc->m->waitingThreads.erase(iter);
+            break;
+          }
+        }
+        pthread_mutex_unlock(&rc->m->thread_list_mutex);
+        pthread_mutex_unlock(cancel_mutex);
+
+        delete rc;
+        pthread_exit(NULL);
+      }
+
+      pthread_mutex_unlock(cancel_mutex);
+    }
+  }
+
+  pthread_mutex_lock(cancel_mutex);
+  pthread_cleanup_push(ChatWaitForSignal_cleanup, arg);
+    pthread_testcancel();
+  pthread_cleanup_pop(0);
+
+  if (rc->m->chatUsers.empty())
+    rc->m->PushChatEvent(new CChatEvent(CHAT_ERRORxCONNECT, NULL));
+
+  pthread_mutex_lock(&rc->m->thread_list_mutex);
+  ThreadList::iterator iter2;
+  for (iter2 = rc->m->waitingThreads.begin();
+                                  iter2 != rc->m->waitingThreads.end(); iter2++)
+  {
+    if (pthread_equal(*iter2, pthread_self()))
+    {
+      rc->m->waitingThreads.erase(iter2);
+      break;
+    }
+  }
+  pthread_mutex_unlock(&rc->m->thread_list_mutex);
+  pthread_mutex_unlock(cancel_mutex);
+
+
+  delete rc->u->m_pClient;
+  delete rc->u;
+  delete rc;
+  
+  pthread_exit(NULL);
+}
+
+void ChatWaitForSignal_cleanup(void *arg)
+{
+  struct SChatReverseConnectInfo *rc = (struct SChatReverseConnectInfo *)arg;
+
+  delete rc->u->m_pClient;
+  delete rc->u;
+  delete rc;
+  pthread_mutex_unlock(&CChatManager::waiting_thread_cancel_mutex);
+}
 
 CChatManager *CChatManager::FindByPort(unsigned short p)
 {
+  pthread_mutex_lock(&cmList_mutex);
   ChatManagerList::iterator iter;
+  CChatManager *cm = NULL;
   for (iter = cmList.begin(); iter != cmList.end(); iter++)
   {
-    if ( (*iter)->LocalPort() == p) return *iter;
+    if ( (*iter)->LocalPort() == p)
+    {
+      cm = *iter;
+      break;
+    }    
   }
-  return NULL;
+  pthread_mutex_unlock(&cmList_mutex);
+  return cm;
 }
 
 
 CChatManager::~CChatManager()
 {
+  // cancel all waiting threads first
+  pthread_mutex_lock(&waiting_thread_cancel_mutex);
+  pthread_mutex_lock(&thread_list_mutex);
+  ThreadList::iterator t_iter;
+  for (t_iter = waitingThreads.begin(); t_iter != waitingThreads.end();)
+  {
+    pthread_cancel(*t_iter);
+    t_iter = waitingThreads.erase(t_iter);
+  }
+  pthread_mutex_unlock(&thread_list_mutex);
+  pthread_mutex_unlock(&waiting_thread_cancel_mutex);
+
   CloseChat();
 
   // Delete all the users
@@ -2263,11 +2610,15 @@ CChatManager::~CChatManager()
     chatEvents.pop_front();
   }
 
+  pthread_mutex_lock(&cmList_mutex);
   ChatManagerList::iterator iter;
   for (iter = cmList.begin(); iter != cmList.end(); iter++)
   {
     if (*iter == this) break;
   }
   if (iter != cmList.end()) cmList.erase(iter);
+  pthread_mutex_unlock(&cmList_mutex);
+  
+  if (m_szId) free(m_szId);
 }
 
