@@ -18,6 +18,9 @@
 #include <fcntl.h>
 #include <string.h>
 
+// Localization
+#include "gettext.h"
+
 #include "licq_filetransfer.h"
 #include "licq_log.h"
 #include "licq_constants.h"
@@ -155,6 +158,8 @@ CFileTransferEvent::CFileTransferEvent(unsigned char t, char *d)
 
 FileTransferManagerList CFileTransferManager::ftmList;
 
+pthread_mutex_t CFileTransferManager::thread_cancel_mutex
+                                                   = PTHREAD_MUTEX_INITIALIZER;
 
 CFileTransferManager::CFileTransferManager(CICQDaemon *d, unsigned long nUin)
 {
@@ -177,6 +182,7 @@ CFileTransferManager::CFileTransferManager(CICQDaemon *d, unsigned long nUin)
   m_nStartTime = m_nBatchStartTime = 0;
   m_nFileDesc = -1;
   m_nState = FT_STATE_DISCONNECTED;
+  m_bThreadCreated = false;
 
   m_szFileName[0] = m_szPathName[0] = '\0';
   sprintf(m_szRemoteName, "%lu", m_nUin);
@@ -190,7 +196,7 @@ bool CFileTransferManager::StartFileTransferServer()
 {
   if (licqDaemon->StartTCPServer(&ftServer) == -1)
   {
-    gLog.Warn("%sNo more ports available, add more or close open chat/file sessions.\n", L_WARNxSTR);
+    gLog.Warn(tr("%sNo more ports available, add more or close open chat/file sessions.\n"), L_WARNxSTR);
     return false;
   }
 
@@ -214,7 +220,7 @@ bool CFileTransferManager::ReceiveFiles(const char *szDirectory)
     if (access(BASE_DIR, F_OK) < 0 && mkdir(m_szDirectory, 0700) == -1 &&
         errno != EEXIST)
     {
-      gLog.Warn("%sUnable to create directory %s for file transfer.\n",
+      gLog.Warn(tr("%sUnable to create directory %s for file transfer.\n"),
          L_WARNxSTR, m_szDirectory);
       strncpy(m_szDirectory, BASE_DIR, MAX_FILENAME_LEN - 1);
       m_szDirectory[MAX_FILENAME_LEN - 1] = '\0';
@@ -230,7 +236,7 @@ bool CFileTransferManager::ReceiveFiles(const char *szDirectory)
   stat(m_szDirectory, &buf);
   if (!S_ISDIR(buf.st_mode))
   {
-    gLog.Warn("%sPath %s is not a directory.\n", L_WARNxSTR, m_szDirectory);
+    gLog.Warn(tr("%sPath %s is not a directory.\n"), L_WARNxSTR, m_szDirectory);
     return false;
   }
 
@@ -247,6 +253,8 @@ bool CFileTransferManager::ReceiveFiles(const char *szDirectory)
     return false;
   }
 
+  m_bThreadCreated = true;
+
   return true;
 }
 
@@ -265,7 +273,7 @@ void CFileTransferManager::SendFiles(ConstFileList lPathNames, unsigned short nP
   {
     if (stat(*iter, &buf) == -1)
     {
-      gLog.Warn("%sFile Transfer: File access error %s:\n%s%s.\n", L_WARNxSTR,
+      gLog.Warn(tr("%sFile Transfer: File access error %s:\n%s%s.\n"), L_WARNxSTR,
          *iter, L_BLANKxSTR, strerror(errno));
       continue;
     }
@@ -277,25 +285,75 @@ void CFileTransferManager::SendFiles(ConstFileList lPathNames, unsigned short nP
   strcpy(m_szPathName, *m_iPathName);
   m_nPort = nPort;
 
+  // start the server anyway, may need to do reverse connect
+  if (!StartFileTransferServer())
+  {
+    PushFileTransferEvent(FT_ERRORxBIND);
+    return;
+  }
+
   // Create the socket manager thread
   if (pthread_create(&thread_ft, NULL, &FileTransferManager_tep, this) == -1)
   {
     PushFileTransferEvent(FT_ERRORxRESOURCES);
     return;
   }
+
+  m_bThreadCreated = true;
 }
 
 
 //-----CFileTransferManager::ConnectToFileServer-----------------------------
 bool CFileTransferManager::ConnectToFileServer(unsigned short nPort)
 {
-  gLog.Info("%sFile Transfer: Connecting to server.\n", L_TCPxSTR);
-  if (!licqDaemon->OpenConnectionToUser(m_nUin, &ftSock, nPort))
-  {
+  ICQUser *u = gUserManager.FetchUser(m_nUin, LOCK_R);
+  if (u == NULL)
     return false;
-  }
 
-  gLog.Info("%sFile Transfer: Shaking hands.\n", L_TCPxSTR);
+  bool bTryDirect = u->Version() <= 6 || u->Mode() == MODE_DIRECT;
+  bool bSendIntIp = u->SendIntIp();
+  gUserManager.DropUser(u);
+
+  bool bSuccess = false;
+  if (bTryDirect)
+  {
+    gLog.Info("%sFile Transfer: Connecting to server.\n", L_TCPxSTR);
+    bSuccess = licqDaemon->OpenConnectionToUser(m_nUin, &ftSock, nPort);
+   }
+ 
+  bool bResult = false;
+  if (!bSuccess)
+  {
+    ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+    unsigned long nIp = bSendIntIp ? o->IntIp() : o->Ip();
+    gUserManager.DropOwner();
+
+    // try reverse connect
+    int nId = licqDaemon->RequestReverseConnection(m_nUin, 0, nIp, LocalPort(),
+                                                                        nPort);
+
+    if (nId != -1)
+    {
+      struct SFileReverseConnectInfo *r = new struct SFileReverseConnectInfo;
+      r->nId = nId;
+      r->m = this;
+      r->bTryDirect = !bTryDirect;
+      pthread_mutex_lock(&thread_cancel_mutex);
+      pthread_create(&m_tThread, NULL, FileWaitForSignal_tep, r);
+      m_bThreadRunning = true;
+      pthread_mutex_unlock(&thread_cancel_mutex);
+      bResult = true;
+    }
+  }
+  else
+    bResult = SendFileHandshake();
+
+  return bResult;
+}
+
+bool CFileTransferManager::SendFileHandshake()
+{
+  gLog.Info(tr("%sFile Transfer: Shaking hands.\n"), L_TCPxSTR);
 
   // Send handshake packet:
   ICQUser *u = gUserManager.FetchUser(m_nUin, LOCK_R);
@@ -308,7 +366,7 @@ bool CFileTransferManager::ConnectToFileServer(unsigned short nPort)
   CPFile_InitClient p(m_szLocalName, m_nBatchFiles, m_nBatchSize);
   if (!SendPacket(&p)) return false;
 
-  gLog.Info("%sFile Transfer: Waiting for server to respond.\n", L_TCPxSTR);
+  gLog.Info(tr("%sFile Transfer: Waiting for server to respond.\n"), L_TCPxSTR);
 
   m_nState = FT_STATE_WAITxFORxSERVERxINIT;
 
@@ -324,7 +382,7 @@ void CFileTransferManager::AcceptReverseConnection(TCPSocket *s)
 {
   if (ftSock.Descriptor() != -1)
   {
-    gLog.Warn("%sFile Transfer: Attempted reverse connection when already connected.\n",
+    gLog.Warn(tr("%sFile Transfer: Attempted reverse connection when already connected.\n"),
        L_WARNxSTR);
     return;
   }
@@ -338,7 +396,7 @@ void CFileTransferManager::AcceptReverseConnection(TCPSocket *s)
   // Reload socket info
   write(pipe_thread[PIPE_WRITE], "R", 1);
 
-  gLog.Info("%sFile Transfer: Received reverse connection.\n", L_TCPxSTR);
+  gLog.Info(tr("%sFile Transfer: Received reverse connection.\n"), L_TCPxSTR);
 }
 
 
@@ -349,9 +407,9 @@ bool CFileTransferManager::ProcessPacket()
   {
     char buf[128];
     if (ftSock.Error() == 0)
-      gLog.Info("%sFile Transfer: Remote end disconnected.\n", L_TCPxSTR);
+      gLog.Info(tr("%sFile Transfer: Remote end disconnected.\n"), L_TCPxSTR);
     else
-      gLog.Warn("%sFile Transfer: Lost remote end:\n%s%s\n", L_WARNxSTR,
+      gLog.Warn(tr("%sFile Transfer: Lost remote end:\n%s%s\n"), L_WARNxSTR,
                 L_BLANKxSTR, ftSock.ErrorStr(buf, 128));
     if (m_nState == FT_STATE_WAITxFORxFILExINFO)
       m_nResult = FT_DONExBATCH;
@@ -369,8 +427,55 @@ bool CFileTransferManager::ProcessPacket()
 
     case FT_STATE_HANDSHAKE:
     {
+      CBuffer tmp(b); // we need to save a copy for later
+      
       if (!CICQDaemon::Handshake_Recv(&ftSock, LocalPort(), false)) break;
-      gLog.Info("%sFile Transfer: Received handshake.\n", L_TCPxSTR);
+      gLog.Info(tr("%sFile Transfer: Received handshake.\n"), L_TCPxSTR);
+      
+      unsigned long nId = 0;
+      if (ftSock.Version() == 7 || ftSock.Version() == 8)
+      {
+        CPacketTcp_Handshake_v7 hand(&tmp);
+        nId = hand.Id();
+      }
+
+      if (nId != 0)
+      {
+        pthread_mutex_lock(&licqDaemon->mutex_reverseconnect);
+        std::list<CReverseConnectToUserData *>::iterator iter;
+        bool bFound = false;
+        for (iter = licqDaemon->m_lReverseConnect.begin();
+                          iter != licqDaemon->m_lReverseConnect.end();  iter++)
+        {
+          if ((*iter)->nId == nId && (*iter)->nUin == m_nUin)
+          {
+            bFound = true;
+            (*iter)->bSuccess = true;
+            (*iter)->bFinished = true;
+            pthread_cond_broadcast(&licqDaemon->cond_reverseconnect_done);
+            break;
+          }
+        }
+        pthread_mutex_unlock(&licqDaemon->mutex_reverseconnect);
+
+        if (bFound)
+        {
+          // Send init packet:
+          CPFile_InitClient p(m_szLocalName, m_nBatchFiles, m_nBatchSize);
+          if (!SendPacket(&p))
+          {
+            m_nResult = FT_ERRORxCLOSED;
+            return false;
+          }
+
+          gLog.Info("%sFile Transfer: Waiting for server to respond.\n",
+            L_TCPxSTR);
+
+          m_nState = FT_STATE_WAITxFORxSERVERxINIT;
+          break;
+        }
+      }
+
       m_nState = FT_STATE_WAITxFORxCLIENTxINIT;
       break;
     }
@@ -381,7 +486,7 @@ bool CFileTransferManager::ProcessPacket()
       if (nCmd == 0x05)
       {
         unsigned long nSpeed = b.UnpackUnsignedLong();
-        gLog.Info("%sFile Transfer: Speed set to %ld%%.\n", L_TCPxSTR, nSpeed);
+        gLog.Info(tr("%sFile Transfer: Speed set to %ld%%.\n"), L_TCPxSTR, nSpeed);
         break;
       }
       if (nCmd != 0x00)
@@ -420,7 +525,7 @@ bool CFileTransferManager::ProcessPacket()
         return false;
       }
 
-      gLog.Info("%sFile Transfer: Waiting for file info.\n", L_TCPxSTR);
+      gLog.Info(tr("%sFile Transfer: Waiting for file info.\n"), L_TCPxSTR);
       m_nState = FT_STATE_WAITxFORxFILExINFO;
       break;
     }
@@ -431,12 +536,12 @@ bool CFileTransferManager::ProcessPacket()
       if (nCmd == 0x05)
       {
         unsigned long nSpeed = b.UnpackUnsignedLong();
-        gLog.Info("%sFile Transfer: Speed set to %ld%%.\n", L_TCPxSTR, nSpeed);
+        gLog.Info(tr("%sFile Transfer: Speed set to %ld%%.\n"), L_TCPxSTR, nSpeed);
         break;
       }
       if (nCmd == 0x06 && b.getDataSize() == 1)
       {
-        gLog.Info("%sFile Transfer: Ignoring a possible erroneous packet.\n",
+        gLog.Info(tr("%sFile Transfer: Ignoring a possible erroneous packet.\n"),
                   L_WARNxSTR);
         break;
       }
@@ -474,7 +579,7 @@ bool CFileTransferManager::ProcessPacket()
       m_nBytesTransfered = 0;
       m_nCurrentFile++;
       
-      gLog.Info("File Transfer: Waiting for plugin to confirm file receive.\n");
+      gLog.Info(tr("File Transfer: Waiting for plugin to confirm file receive.\n"));
       
       m_nState = FT_STATE_CONFIRMINGxFILE;
       PushFileTransferEvent(new CFileTransferEvent(FT_CONFIRMxFILE, m_szPathName));
@@ -484,7 +589,7 @@ bool CFileTransferManager::ProcessPacket()
     case FT_STATE_CONFIRMINGxFILE:
     {
       // Still waiting for the plugin to confirm
-      gLog.Warn("File Transfer: Still waiting for the plugin to confirm file receive...");
+      gLog.Warn(tr("File Transfer: Still waiting for the plugin to confirm file receive..."));
       break;
     }
 
@@ -495,7 +600,7 @@ bool CFileTransferManager::ProcessPacket()
       {
         m_nStartTime = time(TIME_NOW);
         m_nBatchPos += m_nFilePos;
-        gLog.Info("%sFile Transfer: Receiving %s (%ld bytes).\n", L_TCPxSTR,
+        gLog.Info(tr("%sFile Transfer: Receiving %s (%ld bytes).\n"), L_TCPxSTR,
            m_szFileName, m_nFileSize);
         PushFileTransferEvent(new CFileTransferEvent(FT_STARTxFILE, m_szPathName));
         gettimeofday(&tv_lastupdate, NULL);
@@ -507,13 +612,13 @@ bool CFileTransferManager::ProcessPacket()
       if (nCmd == 0x05)
       {
         unsigned long nSpeed = b.UnpackUnsignedLong();
-        gLog.Info("%sFile Transfer: Speed set to %ld%%.\n", L_TCPxSTR, nSpeed);
+        gLog.Info(tr("%sFile Transfer: Speed set to %ld%%.\n"), L_TCPxSTR, nSpeed);
         break;
       }
 
       if (nCmd != 0x06)
       {
-        gLog.Unknown("%sFile Transfer: Invalid data (%c) ignoring packet.\n",
+        gLog.Unknown(tr("%sFile Transfer: Invalid data (%c) ignoring packet.\n"),
            L_UNKNOWNxSTR, nCmd);
         break;
       }
@@ -553,12 +658,12 @@ bool CFileTransferManager::ProcessPacket()
       m_nFileDesc = -1;
       if (nBytesLeft == 0) // File transfer done perfectly
       {
-        gLog.Info("%sFile Transfer: %s received.\n", L_TCPxSTR, m_szFileName);
+        gLog.Info(tr("%sFile Transfer: %s received.\n"), L_TCPxSTR, m_szFileName);
       }
       else // nBytesLeft < 0
       {
         // Received too many bytes for the given size of the current file
-        gLog.Warn("%sFile Transfer: %s received %d too many bytes.\n", L_WARNxSTR,
+        gLog.Warn(tr("%sFile Transfer: %s received %d too many bytes.\n"), L_WARNxSTR,
            m_szFileName, -nBytesLeft);
       }
       // Notify Plugin
@@ -578,7 +683,7 @@ bool CFileTransferManager::ProcessPacket()
       if (nCmd == 0x05)
       {
         unsigned long nSpeed = b.UnpackUnsignedLong();
-        gLog.Info("%sFile Transfer: Speed set to %ld%%.\n", L_TCPxSTR, nSpeed);
+        gLog.Info(tr("%sFile Transfer: Speed set to %ld%%.\n"), L_TCPxSTR, nSpeed);
         break;
       }
       if (nCmd != 0x01)
@@ -597,7 +702,7 @@ bool CFileTransferManager::ProcessPacket()
       CPFile_Info p(*m_iPathName);
       if (!p.IsValid())
       {
-        gLog.Warn("%sFile Transfer: Read error for %s:\n%s\n", L_WARNxSTR,
+        gLog.Warn(tr("%sFile Transfer: Read error for %s:\n%s\n"), L_WARNxSTR,
            *m_iPathName, p.ErrorStr());
         m_nResult = FT_ERRORxFILE;
         return false;
@@ -627,7 +732,7 @@ bool CFileTransferManager::ProcessPacket()
       if (nCmd == 0x05)
       {
         unsigned long nSpeed = b.UnpackUnsignedLong();
-        gLog.Info("%sFile Transfer: Speed set to %ld%%.\n", L_TCPxSTR, nSpeed);
+        gLog.Info(tr("%sFile Transfer: Speed set to %ld%%.\n"), L_TCPxSTR, nSpeed);
         break;
       }
       if (nCmd != 0x03)
@@ -672,11 +777,11 @@ bool CFileTransferManager::ProcessPacket()
       if (nCmd == 0x05)
       {
         unsigned long nSpeed = b.UnpackUnsignedLong();
-        gLog.Info("%sFile Transfer: Speed set to %ld%%.\n", L_TCPxSTR, nSpeed);
+        gLog.Info(tr("%sFile Transfer: Speed set to %ld%%.\n"), L_TCPxSTR, nSpeed);
         break;
       }
       char *p;
-      gLog.Unknown("%sFile Transfer: Unknown packet received during file send:\n%s\n",
+      gLog.Unknown(tr("%sFile Transfer: Unknown packet received during file send:\n%s\n"),
          L_UNKNOWNxSTR, b.print(p));
       delete [] p;
       break;
@@ -701,7 +806,7 @@ bool CFileTransferManager::ProcessPacket()
 // the actual transfer begins
 bool CFileTransferManager::StartReceivingFile(char *szFileName)
 {
-  gLog.Info("%sFile Transfer: Received plugin confirmation.\n", L_TCPxSTR);
+  gLog.Info(tr("%sFile Transfer: Received plugin confirmation.\n"), L_TCPxSTR);
       
   if (m_nState != FT_STATE_CONFIRMINGxFILE)
   {
@@ -770,7 +875,7 @@ bool CFileTransferManager::SendFilePacket()
   {
     m_nStartTime = time(TIME_NOW);
     m_nBatchPos += m_nFilePos;
-    gLog.Info("%sFile Transfer: Sending %s (%ld bytes).\n", L_TCPxSTR,
+    gLog.Info(tr("%sFile Transfer: Sending %s (%ld bytes).\n"), L_TCPxSTR,
        m_szPathName, m_nFileSize);
     PushFileTransferEvent(new CFileTransferEvent(FT_STARTxFILE, m_szPathName));
     gettimeofday(&tv_lastupdate, NULL);
@@ -825,11 +930,11 @@ bool CFileTransferManager::SendFilePacket()
 
   if (nBytesLeft == 0)
   {
-    gLog.Info("%sFile Transfer: Sent %s.\n", L_TCPxSTR, m_szFileName);
+    gLog.Info(tr("%sFile Transfer: Sent %s.\n"), L_TCPxSTR, m_szFileName);
   }
   else // nBytesLeft < 0
   {
-    gLog.Info("%sFile Transfer: Sent %s, %d too many bytes.\n", L_TCPxSTR,
+    gLog.Info(tr("%sFile Transfer: Sent %s, %d too many bytes.\n"), L_TCPxSTR,
        m_szFileName, -nBytesLeft);
   }
   PushFileTransferEvent(new CFileTransferEvent(FT_DONExFILE, m_szPathName));
@@ -847,7 +952,7 @@ bool CFileTransferManager::SendFilePacket()
     CPFile_Info p(*m_iPathName);
     if (!p.IsValid())
     {
-      gLog.Warn("%sFile Transfer: Read error for %s:\n%s\n", L_WARNxSTR,
+      gLog.Warn(tr("%sFile Transfer: Read error for %s:\n%s\n"), L_WARNxSTR,
          *m_iPathName, p.ErrorStr());
       m_nResult = FT_ERRORxFILE;
       return false;
@@ -870,7 +975,7 @@ bool CFileTransferManager::SendFilePacket()
 
 
 
-//-----CFileTransferManager::PopChatEvent------------------------------------
+//-----CFileTransferManager::PopFileTransferEvent------------------------------
 CFileTransferEvent *CFileTransferManager::PopFileTransferEvent()
 {
   if (ftEvents.size() == 0) return NULL;
@@ -882,7 +987,7 @@ CFileTransferEvent *CFileTransferManager::PopFileTransferEvent()
 }
 
 
-//-----CFileTransferManager::PushChatEvent-------------------------------------------
+//-----CFileTransferManager::PushFileTransferEvent-----------------------------
 void CFileTransferManager::PushFileTransferEvent(unsigned char t)
 {
   PushFileTransferEvent(new CFileTransferEvent(t));
@@ -908,7 +1013,7 @@ bool CFileTransferManager::SendBuffer(CBuffer *b)
   if (!ftSock.SendPacket(b))
   {
     char buf[128];
-    gLog.Warn("%sFile Transfer: Send error:\n%s%s\n", L_WARNxSTR, L_BLANKxSTR,
+    gLog.Warn(tr("%sFile Transfer: Send error:\n%s%s\n"), L_WARNxSTR, L_BLANKxSTR,
        ftSock.ErrorStr(buf, 128));
     return false;
   }
@@ -920,7 +1025,7 @@ void CFileTransferManager::ChangeSpeed(unsigned short nSpeed)
 {
   if (nSpeed > 100)
   {
-    gLog.Warn("%sInvalid file transfer speed: %d%%.\n", L_WARNxSTR, nSpeed);
+    gLog.Warn(tr("%sInvalid file transfer speed: %d%%.\n"), L_WARNxSTR, nSpeed);
     return;
   }
 
@@ -938,8 +1043,10 @@ void CFileTransferManager::CloseFileTransfer()
   if (pipe_thread[PIPE_WRITE] != -1)
   {
     write(pipe_thread[PIPE_WRITE], "X", 1);
-    pthread_join(thread_ft, NULL);
-
+    if (m_bThreadCreated)
+      pthread_join(thread_ft, NULL);
+    m_bThreadCreated = false;
+   
     close(pipe_thread[PIPE_READ]);
     close(pipe_thread[PIPE_WRITE]);
 
@@ -1031,7 +1138,7 @@ void *FileTransferManager_tep(void *arg)
         if (nCurrentSocket == ftman->pipe_thread[PIPE_READ])
         {
           read(ftman->pipe_thread[PIPE_READ], buf, 1);
-          if (buf[0] == 'S')
+          if (buf[0] == 'R')
           {
             DEBUG_THREADS("[FileTransferManager_tep] Reloading socket info.\n");
           }
@@ -1047,7 +1154,7 @@ void *FileTransferManager_tep(void *arg)
         {
           if (ftman->ftSock.Descriptor() != -1)
           {
-            gLog.Warn("%sFile Transfer: Receiving repeat incoming connection.\n", L_WARNxSTR);
+            gLog.Warn(tr("%sFile Transfer: Receiving repeat incoming connection.\n"), L_WARNxSTR);
           }
           else
           {
@@ -1056,7 +1163,7 @@ void *FileTransferManager_tep(void *arg)
             ftman->sockman.DropSocket(&ftman->ftSock);
 
             ftman->m_nState = FT_STATE_HANDSHAKE;
-            gLog.Info("%sFile Transfer: Received connection.\n", L_TCPxSTR);
+            gLog.Info(tr("%sFile Transfer: Received connection.\n"), L_TCPxSTR);
           }
         }
 
@@ -1075,7 +1182,7 @@ void *FileTransferManager_tep(void *arg)
 
         else
         {
-          gLog.Warn("%sFile Transfer: No such socket.\n", L_WARNxSTR);
+          gLog.Warn(tr("%sFile Transfer: No such socket.\n"), L_WARNxSTR);
         }
 
         nSocketsAvailable--;
@@ -1102,6 +1209,93 @@ void *FileTransferManager_tep(void *arg)
   return NULL;
 }
 
+void *FileWaitForSignal_tep(void *arg)
+{
+  pthread_detach(pthread_self());
+
+  CICQDaemon *d;
+  unsigned long nUin;
+  unsigned short nPort;
+  struct SFileReverseConnectInfo *rc = (struct SFileReverseConnectInfo *)arg;
+  pthread_mutex_t *cancel_mutex = &CFileTransferManager::thread_cancel_mutex;
+
+  pthread_mutex_lock(cancel_mutex);
+  pthread_cleanup_push(FileWaitForSignal_cleanup, arg);
+    pthread_testcancel();
+  pthread_cleanup_pop(0);
+  d = rc->m->licqDaemon;
+  nUin = rc->m->Uin();
+  nPort = rc->m->m_nPort;
+  pthread_mutex_unlock(cancel_mutex);
+
+  pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
+  gLog.Info("%sFile Transfer: Waiting for reverse connection.\n", L_TCPxSTR);
+  bool bConnected = d->WaitForReverseConnection(rc->nId, nUin);
+  pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
+
+  pthread_mutex_lock(cancel_mutex);
+  pthread_cleanup_push(FileWaitForSignal_cleanup, arg);
+    pthread_testcancel();
+  pthread_cleanup_pop(0);
+
+  if (bConnected || !rc->bTryDirect)
+  {
+    if (!bConnected)
+      rc->m->PushFileTransferEvent(FT_ERRORxCONNECT);
+
+    rc->m->m_bThreadRunning = false;
+    pthread_mutex_unlock(cancel_mutex);
+
+    delete rc;
+    pthread_exit(NULL);
+  }
+
+  pthread_mutex_unlock(cancel_mutex);
+
+  pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
+  gLog.Info("%sFile Transfer: Reverse connection failed, trying direct.\n",
+                                                                    L_TCPxSTR);
+  TCPSocket s;
+  bConnected = d->OpenConnectionToUser(nUin, &s, nPort);
+  pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
+
+  pthread_mutex_lock(cancel_mutex);
+  pthread_cleanup_push(FileWaitForSignal_cleanup, arg);
+    pthread_testcancel();
+  pthread_cleanup_pop(0);
+
+  if (bConnected)
+  {
+    if (rc->m->ftSock.Descriptor() != -1)
+    {
+      gLog.Warn("%sFile Transfer: Attempted connection when already"
+                " connected.\n", L_WARNxSTR);
+    }
+    else
+    {
+      rc->m->ftSock.TransferConnectionFrom(s);
+      bConnected = rc->m->SendFileHandshake();
+      write(rc->m->pipe_thread[PIPE_WRITE], "R", 1);
+    }
+  }
+
+  if (!bConnected)
+    rc->m->PushFileTransferEvent(FT_ERRORxCONNECT);
+
+  rc->m->m_bThreadRunning = false;
+
+  pthread_mutex_unlock(cancel_mutex);
+  delete rc;
+  pthread_exit(NULL);
+}
+
+void FileWaitForSignal_cleanup(void *arg)
+{
+  struct SFileReverseConnectInfo *rc = (struct SFileReverseConnectInfo *)arg;
+  delete rc;
+
+  pthread_mutex_unlock(&CFileTransferManager::thread_cancel_mutex);
+}
 
 CFileTransferManager *CFileTransferManager::FindByPort(unsigned short p)
 {
@@ -1116,6 +1310,12 @@ CFileTransferManager *CFileTransferManager::FindByPort(unsigned short p)
 
 CFileTransferManager::~CFileTransferManager()
 {
+  // cancel the waiting thread first
+  pthread_mutex_lock(&thread_cancel_mutex);
+  if (m_bThreadRunning)
+    pthread_cancel(m_tThread);
+  pthread_mutex_unlock(&thread_cancel_mutex);
+
   CloseFileTransfer();
 
   // Delete any pending events
