@@ -20,6 +20,9 @@
 #include "licq_plugind.h"
 #include "licq.h"
 
+// Localization
+#include "gettext.h"
+
 #define DEBUG_THREADS(x)
 //#define DEBUG_THREADS(x) gLog.Info(x)
 
@@ -152,7 +155,8 @@ void *ProcessRunningEvent_Server_tep(void *p)
 
   // declared here because pthread_cleanup_push starts a new block
   CBuffer *buf;
-  bool sent;
+  bool sent = false;
+  bool bExit = false;
   char szErrorBuf[128];
 
   pthread_cleanup_push(cleanup_mutex, &send_mutex);
@@ -160,6 +164,10 @@ void *ProcessRunningEvent_Server_tep(void *p)
     pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
     pthread_testcancel();
 
+    int socket = -1;
+    unsigned long nSequence;
+    INetSocket *s;
+    
     // Check if the socket is connected
     if (e->m_nSocketDesc == -1)
     {
@@ -167,9 +175,8 @@ void *ProcessRunningEvent_Server_tep(void *p)
       if (e->m_pPacket->Channel() == ICQ_CHNxNEW)
       {
         pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
-        gLog.Info("%sConnecting to login server.\n", L_SRVxSTR);
+        gLog.Info(tr("%sConnecting to login server.\n"), L_SRVxSTR);
 
-/***** Not yet, perhaps a problem with certain versions of pthreads? *****
         pthread_t *t = new pthread_t;
         int *s = new int;
         pthread_create(t, NULL, ConnectToServer_tep, s);
@@ -185,10 +192,6 @@ void *ProcessRunningEvent_Server_tep(void *p)
 
         pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
         pthread_testcancel();
-*********************/
-
-        int socket = d->ConnectToLoginServer();
-        pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
 
         e->m_nSocketDesc = socket;
 
@@ -196,7 +199,7 @@ void *ProcessRunningEvent_Server_tep(void *p)
         if (e->m_nSocketDesc == -1)
         {
           pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
-          gLog.Info("%sConnecting to login server failed, failing event\n",
+          gLog.Info(tr("%sConnecting to login server failed, failing event\n"),
                     L_SRVxSTR);
           // we need to initialize the logon time for the next retry
           d->m_tLogonTime = time(NULL);
@@ -213,13 +216,14 @@ void *ProcessRunningEvent_Server_tep(void *p)
             pthread_testcancel();
             delete e;
           }
-          pthread_exit(NULL);
+          bExit = true;
+          goto exit_server_thread;
         }
       }
       else
       {
         pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
-        gLog.Info("%sNot connected to server, failing event\n", L_SRVxSTR);
+        gLog.Info(tr("%sNot connected to server, failing event\n"), L_SRVxSTR);
         if (d->DoneEvent(e, EVENT_ERROR) != NULL)
         {
           d->DoneExtendedEvent(e, EVENT_ERROR);
@@ -231,19 +235,20 @@ void *ProcessRunningEvent_Server_tep(void *p)
           pthread_testcancel();
           delete e;
         }
-        pthread_exit(NULL);
+        bExit = true;
+        goto exit_server_thread;
       }
     }
 
-    int socket = e->m_nSocketDesc;
-    unsigned long nSequence = e->m_nSequence;
+    socket = e->m_nSocketDesc;
+    nSequence = e->m_nSequence;
 
     pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
     // Start sending the event
-    INetSocket *s = gSocketManager.FetchSocket(socket);
+    s = gSocketManager.FetchSocket(socket);
     if (s == NULL)
     {
-      gLog.Warn("%sSocket not connected or invalid (#%lu).\n", L_WARNxSTR,
+      gLog.Warn(tr("%sSocket not connected or invalid (#%lu).\n"), L_WARNxSTR,
                 nSequence);
       if (d->DoneEvent(e, EVENT_ERROR) != NULL)
       {
@@ -256,28 +261,27 @@ void *ProcessRunningEvent_Server_tep(void *p)
         pthread_testcancel();
         delete e;
       }
-      pthread_exit(NULL);
+      bExit = true;
+      goto exit_server_thread;
     }
 
     pthread_cleanup_push(cleanup_socket, s);
 
-      /* FIXME ugly hack, but prevents event from being deleted while we still
-         need it and cannot be canceled */
-      pthread_mutex_lock(&d->mutex_runningevents);
+      pthread_mutex_lock(&d->mutex_cancelthread);
 
       // check to make sure we were not cancelled already
-      pthread_cleanup_push(cleanup_mutex, &d->mutex_runningevents);
+      pthread_cleanup_push(cleanup_mutex, &d->mutex_cancelthread);
         pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
         pthread_testcancel();
         pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
 
         //if we get here  then we haven't been cancelled and we won't be
-        //as long as we hold mutex_runningevents
+        //as long as we hold mutex_cancelthread
 
         buf = e->m_pPacket->Finalize(NULL);
 
-        pthread_mutex_unlock(&d->mutex_runningevents);
-      pthread_cleanup_pop(0); //mutex_runningevents
+        pthread_mutex_unlock(&d->mutex_cancelthread);
+      pthread_cleanup_pop(0); //mutex_cancelthread
 
       sent = s->Send(buf);
       delete buf;
@@ -291,9 +295,13 @@ void *ProcessRunningEvent_Server_tep(void *p)
       gSocketManager.DropSocket(s);
     pthread_cleanup_pop(0); //socket
 
+exit_server_thread:
     pthread_mutex_unlock(&send_mutex);
   pthread_cleanup_pop(0); //send_mutex
 
+  if (bExit)
+    pthread_exit(NULL);
+    
   pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
   pthread_testcancel();
 
@@ -303,7 +311,7 @@ void *ProcessRunningEvent_Server_tep(void *p)
 
     pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
 
-    gLog.Warn("%sError sending event (#%lu):\n%s%s.\n", L_WARNxSTR,
+    gLog.Warn(tr("%sError sending event (#%lu):\n%s%s.\n"), L_WARNxSTR,
               nSequence, L_BLANKxSTR, szErrorBuf);
 
     if (d->DoneEvent(e, EVENT_ERROR) != NULL)
@@ -372,8 +380,104 @@ void *ProcessRunningEvent_Client_tep(void *p)
   if (e->m_nSocketDesc == -1)
   {
     unsigned long nDestinationUin = e->m_nDestinationUin;
+    unsigned char nChannel = e->Channel();
     pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
-    int socket = d->ConnectToUser(nDestinationUin);
+   
+    ICQUser *u = gUserManager.FetchUser(nDestinationUin, LOCK_R);
+    if (u == NULL)
+    {
+      if (d->DoneEvent(e, EVENT_ERROR) != NULL)
+        d->ProcessDoneEvent(e);
+      else
+      {
+        pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
+        pthread_testcancel();
+        delete e;
+      }
+      pthread_exit(NULL);
+    }
+
+    unsigned long nVersion = u->Version();
+    unsigned char nMode = u->Mode();
+    unsigned short nRemotePort = u->Port();
+    bool bSendIntIp = u->SendIntIp();
+    gUserManager.DropUser(u);
+
+    ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+    unsigned long nIP = bSendIntIp ? o->IntIp() : o->Ip();
+    unsigned short nLocalPort = o->Port();
+    gUserManager.DropOwner();
+
+    int socket = -1;
+    if (nVersion > 6 && nMode != MODE_DIRECT)
+    {
+      int nId = d->RequestReverseConnection(nDestinationUin, nChannel, nIP,
+                                            nLocalPort, nRemotePort);
+      if (nId != -1)
+      {
+        d->WaitForReverseConnection(nId, nDestinationUin);
+        u = gUserManager.FetchUser(nDestinationUin, LOCK_R);
+        if (u == NULL)
+        {
+          if (d->DoneEvent(e, EVENT_ERROR) != NULL)
+            d->ProcessDoneEvent(e);
+          else
+          {
+            pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
+            pthread_testcancel();
+            delete e;
+          }
+          pthread_exit(NULL);
+        }
+        socket = u->SocketDesc(nChannel);
+        gUserManager.DropUser(u);
+      }
+      
+      // if we failed, try direct anyway
+      if (socket == -1)
+      {
+        pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
+        pthread_testcancel();
+        pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
+
+        socket = d->ConnectToUser(nDestinationUin, nChannel);
+      }
+    }
+    else
+    {
+      socket = d->ConnectToUser(nDestinationUin, nChannel);
+
+      // if we failed, try through server
+      if (socket == -1)
+      {
+        pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
+        pthread_testcancel();
+        pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
+
+        int nId = d->RequestReverseConnection(nDestinationUin, nChannel, nIP,
+                                              nLocalPort, nRemotePort);
+        if (nId != -1)
+        {
+          d->WaitForReverseConnection(nId, nDestinationUin);
+          u = gUserManager.FetchUser(nDestinationUin, LOCK_R);
+          if (u == NULL)
+          {
+            if (d->DoneEvent(e, EVENT_ERROR) != NULL)
+              d->ProcessDoneEvent(e);
+            else
+            {
+              pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
+              pthread_testcancel();
+              delete e;
+            }
+            pthread_exit(NULL);
+          }
+          socket = u->SocketDesc(nChannel);
+          gUserManager.DropUser(u);
+        }
+      }
+    }
+
     pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
     pthread_testcancel();
     e->m_nSocketDesc = socket;
@@ -403,7 +507,7 @@ void *ProcessRunningEvent_Client_tep(void *p)
     unsigned long nSequence = e->m_nSequence;
     pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
 
-    gLog.Warn("%sSocket %d does not exist (#%lu).\n", L_WARNxSTR, socket,
+    gLog.Warn(tr("%sSocket %d does not exist (#%lu).\n"), L_WARNxSTR, socket,
        nSequence);
     if (d->DoneEvent(e, EVENT_ERROR) != NULL)
       d->ProcessDoneEvent(e);
@@ -454,7 +558,7 @@ void *ProcessRunningEvent_Client_tep(void *p)
     unsigned long nSequence = e->m_nSequence;
     pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
 
-    gLog.Warn("%sError sending event (#%lu):\n%s%s.\n", L_WARNxSTR,
+    gLog.Warn(tr("%sError sending event (#%lu):\n%s%s.\n"), L_WARNxSTR,
      -nSequence, L_BLANKxSTR, szErrorBuf);
     write(d->pipe_newsocket[PIPE_WRITE], "S", 1);
     // Kill the event, do after the above as ProcessDoneEvent erase the event
@@ -496,7 +600,7 @@ void *ReverseConnectToUser_tep(void *v)
   CReverseConnectToUserData *p = (CReverseConnectToUserData *)v;
 
   gLicqDaemon->ReverseConnectToUser(p->nUin, p->nIp, p->nPort, p->nVersion,
-    p->nFailedPort);
+    p->nFailedPort, p->nId, p->nMsgID1, p->nMsgID2);
 
   delete p;
 
@@ -629,11 +733,11 @@ void *MonitorSockets_tep(void *p)
           // Message from the server -------------------------------------------
           else if (nCurrentSocket == d->m_nTCPSrvSocketDesc)
           {
-            DEBUG_THREADS("[MonitorSockets_tep] Data on TCP server socket.\n");
-            SrvSocket *srvTCP = static_cast<SrvSocket*>(s);
-            if (srvTCP == NULL)
-            {
-              gLog.Warn("%sInvalid server socket in set.\n", L_WARNxSTR);
+              DEBUG_THREADS("[MonitorSockets_tep] Data on TCP server socket.\n");
+              SrvSocket *srvTCP = static_cast<SrvSocket*>(s);
+              if (srvTCP == NULL)
+              {
+              gLog.Warn(tr("%sInvalid server socket in set.\n"), L_WARNxSTR);
               close(nCurrentSocket);
             }
             // DAW FIXME error handling when socket is closed..
@@ -668,7 +772,7 @@ void *MonitorSockets_tep(void *p)
             TCPSocket *tcp = static_cast<TCPSocket *>(s);
             if (tcp == NULL)
             {
-              gLog.Warn("%sInvalid server TCP socket in set.\n", L_WARNxSTR);
+              gLog.Warn(tr("%sInvalid server TCP socket in set.\n"), L_WARNxSTR);
               close(nCurrentSocket);
             }
             else
@@ -699,12 +803,12 @@ void *MonitorSockets_tep(void *p)
             {
               int err = tcp->Error();
               if (err == 0)
-                gLog.Info("%sConnection to %lu was closed.\n", L_TCPxSTR
+                gLog.Info(tr("%sConnection to %lu was closed.\n"), L_TCPxSTR
                                                              , tcp->Owner());
               else
               {
                 char buf[128];
-                gLog.Info("%sConnection to %lu lost:\n%s%s.\n", L_TCPxSTR,
+                gLog.Info(tr("%sConnection to %lu lost:\n%s%s.\n"), L_TCPxSTR,
                           tcp->Owner(), L_BLANKxSTR, tcp->ErrorStr(buf, 128));
               }
               gSocketManager.DropSocket(tcp);
@@ -731,7 +835,7 @@ void *MonitorSockets_tep(void *p)
             // Kill the socket if there was a problem
             if (!r)
             {
-              gLog.Info("%sClosing connection to %lu.\n", L_TCPxSTR,
+              gLog.Info(tr("%sClosing connection to %lu.\n"), L_TCPxSTR,
                                                           tcp->Owner());
               gSocketManager.DropSocket(tcp);
               gSocketManager.CloseSocket(nCurrentSocket);
@@ -774,7 +878,7 @@ void *Shutdown_tep(void *p)
   CICQDaemon *d = (CICQDaemon *)p;
 
   // Shutdown
-  gLog.Info("%sShutting down daemon.\n", L_ENDxSTR);
+  gLog.Info(tr("%sShutting down daemon.\n"), L_ENDxSTR);
 
   // Send shutdown signal to all the plugins
   d->licq->ShutdownPlugins();
@@ -785,6 +889,9 @@ void *Shutdown_tep(void *p)
   // Cancel the ping thread
   pthread_cancel(d->thread_ping);
 
+  // Cancel the update users thread
+  pthread_cancel(d->thread_updateusers);
+  
   // Join our threads
   pthread_join(d->thread_monitorsockets, NULL);
 
@@ -805,5 +912,96 @@ void *Shutdown_tep(void *p)
   return NULL;
 }
 
+void *UpdateUsers_tep(void *p)
+{
+  pthread_detach(pthread_self());
+
+  CICQDaemon *d = (CICQDaemon *)p;
+  struct timeval tv;
+
+  while (true)
+  {
+    if (d->m_eStatus == STATUS_ONLINE)
+    {
+      pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
+      FOR_EACH_PROTO_USER_START(LICQ_PPID, LOCK_R)
+      {
+        bool bSent = false;
+
+        if (d->AutoUpdateInfo() && !pUser->UserUpdated() &&
+            pUser->ClientTimestamp() != pUser->OurClientTimestamp()
+            && pUser->ClientTimestamp() != 0)
+        {
+          d->icqRequestMetaInfo(pUser->Uin());
+          bSent = true;
+        }
+
+        if (!pUser->StatusOffline() && !pUser->UserUpdated() &&
+            //Don't bother clients that we know don't support plugins
+            pUser->Version() >= 7 &&
+            //Old versions of Licq
+            ((pUser->ClientTimestamp() & 0xFFFF0000 != LICQ_WITHSSL &&
+              pUser->ClientTimestamp() & 0xFFFF0000 != LICQ_WITHOUTSSL) ||
+             pUser->ClientTimestamp() & 0xFFFF > 1026) &&
+            pUser->ClientTimestamp() != 0xFFFFFF42 && //mICQ
+            pUser->ClientTimestamp() != 0xFFFFFFFF && //Miranda
+            pUser->ClientTimestamp() != 0xFFFFFF7F && //&RQ
+            pUser->ClientTimestamp() != 0xFFFFFFBE && //Alicq
+            pUser->ClientTimestamp() != 0x3B75AC09 && //Trillian
+            pUser->ClientTimestamp() != 0x3AA773EE && //libICQ2000 based clients
+            pUser->ClientTimestamp() != 0x3BC1252C && //ICQ Interest Search
+            pUser->ClientTimestamp() != 0x3B176B57 && //jcq2k
+            pUser->ClientTimestamp() != 0x3BA76E2E && //SmartICQ
+            pUser->ClientTimestamp() != 0x3C7D8CBC && //Vista
+            pUser->ClientTimestamp() != 0x3CFE0688 && //Meca
+            pUser->ClientTimestamp() != 0x3BFF8C98 //IGA
+           )
+        {
+          if (d->AutoUpdateInfoPlugins() &&
+              pUser->ClientInfoTimestamp() != pUser->OurClientInfoTimestamp() &&
+              pUser->ClientInfoTimestamp() != 0)
+          {
+            gLog.Info("Updating %s's info plugins.\n", pUser->GetAlias());
+            d->icqRequestInfoPlugin(pUser, true, PLUGIN_QUERYxINFO);
+            d->icqRequestInfoPlugin(pUser, true, PLUGIN_PHONExBOOK);
+            d->icqRequestInfoPlugin(pUser, true, PLUGIN_PICTURE);
+            bSent = true;
+          }
+
+          if (d->AutoUpdateStatusPlugins() &&
+             pUser->ClientStatusTimestamp() != pUser->OurClientStatusTimestamp()
+              && pUser->ClientStatusTimestamp() != 0)
+          {
+            gLog.Info("Updating %s's status plugins.\n", pUser->GetAlias());
+            d->icqRequestStatusPlugin(pUser, true, PLUGIN_QUERYxSTATUS);
+            d->icqRequestStatusPlugin(pUser, true, PLUGIN_FILExSERVER);
+            d->icqRequestStatusPlugin(pUser, true, PLUGIN_FOLLOWxME);
+            d->icqRequestStatusPlugin(pUser, true, PLUGIN_ICQxPHONE);
+            bSent = true;
+          }
+
+        }
+
+        if (bSent)
+        {
+          pUser->SetUserUpdated(true);
+          FOR_EACH_PROTO_USER_BREAK;
+        }
+      }
+      FOR_EACH_PROTO_USER_END
+      pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
+    }
+
+    pthread_testcancel();
+
+    tv.tv_sec = UPDATE_FREQUENCY;
+    tv.tv_usec = 0;
+    select(0, NULL, NULL, NULL, &tv);
+
+    pthread_testcancel();
+  }
+
+  pthread_exit(NULL);
+}
 
 #endif
