@@ -14,10 +14,14 @@
 #include <stdlib.h>
 #include <string.h>
 
+// Localization
+#include "gettext.h"
+
 #include "licq_message.h"
 #include "licq_user.h"
 #include "licq_translate.h"
 #include "licq_icqd.h"
+#include "licq_gpg.h"
 #include "support.h"
 
 #ifdef USE_HEBREW
@@ -26,9 +30,8 @@ extern char *hebrev (char* pszStr);
 }
 #endif
 
-#ifdef PROTOCOL_PLUGIN
+// Hack
 extern char *PPIDSTRING(unsigned long);
-#endif
 
 int CUserEvent::s_nId = 1;
 
@@ -131,15 +134,15 @@ int CUserEvent::AddToHistory_Header(direction _nDir, char *szOut)
 }
 
 
-void CUserEvent::AddToHistory_Flush(ICQUser *u, char *szOut)
+void CUserEvent::AddToHistory_Flush(ICQUser *u, char *szOut, unsigned long nPPID)
 {
   if (u != NULL)
     u->WriteToHistory(szOut);
   else
   {
-    ICQOwner *o = gUserManager.FetchOwner(LOCK_W);
+    ICQOwner *o = gUserManager.FetchOwner(nPPID, LOCK_W);
     o->WriteToHistory(szOut);
-    gUserManager.DropOwner();
+    gUserManager.DropOwner(nPPID);
   }
 }
 
@@ -153,6 +156,13 @@ CEventMsg::CEventMsg(const char *_szMessage, unsigned short _nCommand,
    : CUserEvent(ICQ_CMDxSUB_MSG, _nCommand, 0, _tTime, _nFlags)
 {
   m_szMessage = strdup(_szMessage == NULL ? "" : _szMessage);
+  
+  if (strstr(m_szMessage, CGPGHelper::pgpSig) == m_szMessage)
+    if (char *plaintext = gGPGHelper.Decrypt(m_szMessage)) {
+      m_nFlags |= E_ENCRYPTED;
+	  free(m_szMessage);
+	  m_szMessage = plaintext;
+	}
 }
 
 
@@ -212,7 +222,7 @@ void CEventFile::CreateDescription()
 {
   if (m_szText) delete [] m_szText;
   m_szText = new char[strlen(m_szFilename) + strlen(m_szFileDescription) + 64];
-  sprintf(m_szText, "File: %s (%lu bytes)\nDescription:\n%s\n", m_szFilename,
+  sprintf(m_szText, tr("File: %s (%lu bytes)\nDescription:\n%s\n"), m_szFilename,
           m_nFileSize, m_szFileDescription);
 }
 
@@ -252,7 +262,7 @@ void CEventUrl::CreateDescription()
 {
   if (m_szText) delete [] m_szText;
   m_szText = new char[strlen(m_szUrl) + strlen(m_szUrlDescription) + 64];
-  sprintf(m_szText, "Url: %s\nDescription:\n%s\n", m_szUrl, m_szUrlDescription);
+  sprintf(m_szText, tr("Url: %s\nDescription:\n%s\n"), m_szUrl, m_szUrlDescription);
 }
 
 
@@ -332,7 +342,7 @@ void CEventChat::CreateDescription()
   else
   {
     m_szText = new char[strlen(m_szReason) + strlen(m_szClients) + 128];
-    sprintf(m_szText, "%s\n--------------------\nMultiparty:\n%s", m_szReason, m_szClients);
+    sprintf(m_szText, tr("%s\n--------------------\nMultiparty:\n%s"), m_szReason, m_szClients);
   }
 }
 
@@ -355,7 +365,6 @@ void CEventChat::AddToHistory(ICQUser *u, direction _nDir)
 
 
 //=====CEventAdded==============================================================
-#ifdef PROTOCOL_PLUGIN
 CEventAdded::CEventAdded(const char *_szId, unsigned long _nPPID, const char *_szAlias,
                          const char *_szFirstName,const char *_szLastName,
                          const char *_szEmail, unsigned short _nCommand,
@@ -373,7 +382,6 @@ CEventAdded::CEventAdded(const char *_szId, unsigned long _nPPID, const char *_s
   else
     m_nUin = 0;
 }
-#endif
 
 CEventAdded::CEventAdded(unsigned long _nUin, const char *_szAlias,
                          const char *_szFirstName,const char *_szLastName,
@@ -386,33 +394,23 @@ CEventAdded::CEventAdded(unsigned long _nUin, const char *_szAlias,
   m_szLastName = strdup(_szLastName);
   m_szEmail = strdup(_szEmail);
   m_nUin = _nUin;
-#ifdef PROTOCOL_PLUGIN
+
   char szUin[24];
   sprintf(szUin, "%lu", _nUin);
   m_szId = strdup(szUin);
   m_nPPID = LICQ_PPID;
-#endif
 }
 
 void CEventAdded::CreateDescription()
 {
   if (m_szText) delete [] m_szText;
-#ifdef PROTOCOL_PLUGIN
   char *p = PPIDSTRING(m_nPPID);
   m_szText = new char[strlen(m_szAlias) + strlen(m_szFirstName) +
                       strlen(m_szLastName) + strlen(m_szEmail) +
                       strlen(m_szId) + strlen(p) + 512];
-  sprintf(m_szText, "Alias: %s\nUser: %s (%s)\nName: %s %s\nEmail: %s\n",
+  sprintf(m_szText, tr("Alias: %s\nUser: %s (%s)\nName: %s %s\nEmail: %s\n"),
     m_szAlias, m_szId, p, m_szFirstName, m_szLastName, m_szEmail);
   delete [] p;
-#else
-  m_szText = new char[strlen(m_szAlias) + strlen(m_szFirstName) +
-                      strlen(m_szLastName) + strlen(m_szEmail) + 512];
-  //sprintf(m_szText, "%s (%s %s, %s), uin %lu, added you to their contact list.\n",
-  //        m_szAlias, m_szFirstName, m_szLastName, m_szEmail, m_nUin);
-  sprintf(m_szText, "Alias: %s\nUin: %lu\nName: %s %s\nEmail: %s\n",
-     m_szAlias, m_nUin, m_szFirstName, m_szLastName, m_szEmail);
-#endif
 }
 
 
@@ -422,14 +420,11 @@ CEventAdded::~CEventAdded()
   free (m_szFirstName);
   free (m_szLastName);
   free (m_szEmail);
-#ifdef PROTOCOL_PLUGIN
   free (m_szId);
-#endif
 }
 
 void CEventAdded::AddToHistory(ICQUser *u, direction _nDir)
 {
-#ifdef PROTOCOL_PLUGIN
   char *p = PPIDSTRING(m_nPPID);
   char *szOut = new char[(strlen(m_szAlias) + strlen(m_szFirstName) +
                     strlen(m_szLastName) + strlen(m_szEmail) + strlen(m_szId) +
@@ -438,22 +433,13 @@ void CEventAdded::AddToHistory(ICQUser *u, direction _nDir)
   nPos += sprintf(&szOut[nPos], ":%s (%s)\n:%s\n:%s\n:%s\n:%s\n", m_szId,
                   p, m_szAlias, m_szFirstName, m_szLastName, m_szEmail);
   delete [] p;
-#else
-  char *szOut = new char[(strlen(m_szAlias) + strlen(m_szFirstName) +
-                    strlen(m_szLastName) + strlen(m_szEmail)) * 2 + 16 +
-                   EVENT_HEADER_SIZE];
-  int nPos = AddToHistory_Header(_nDir, szOut);
-  nPos += sprintf(&szOut[nPos], ":%lu\n:%s\n:%s\n:%s\n:%s\n", m_nUin,
-                  m_szAlias, m_szFirstName, m_szLastName, m_szEmail);
-#endif
-  AddToHistory_Flush(u, szOut);
+  AddToHistory_Flush(u, szOut, m_nPPID);
   delete [] szOut;
 }
 
 
 
 //=====CEventAuthReq===============================================================
-#ifdef PROTOCOL_PLUGIN
 CEventAuthRequest::CEventAuthRequest(const char *_szId, unsigned long _nPPID,
                        const char *_szAlias, const char *_szFirstName,
                        const char *_szLastName, const char *_szEmail,
@@ -473,7 +459,6 @@ CEventAuthRequest::CEventAuthRequest(const char *_szId, unsigned long _nPPID,
    else
     m_nUin = 0;
 }
-#endif
 
 CEventAuthRequest::CEventAuthRequest(unsigned long _nUin, const char *_szAlias,
                        const char *_szFirstName,const char *_szLastName,
@@ -487,39 +472,29 @@ CEventAuthRequest::CEventAuthRequest(unsigned long _nUin, const char *_szAlias,
    m_szLastName = strdup(_szLastName);
    m_szEmail = strdup(_szEmail);
    m_szReason = strdup(_szReason);
-#ifdef PROTOCOL_PLUGIN
+
    char szUin[24];
    sprintf(szUin, "%lu", _nUin);
    m_szId = strdup(szUin);
    m_nPPID = LICQ_PPID;
-#endif
    m_nUin = _nUin;
 }
 
 void CEventAuthRequest::CreateDescription()
 {
   if (m_szText) delete [] m_szText;
-#ifdef PROTOCOL_PLUGIN
   char *p = PPIDSTRING(m_nPPID);
   m_szText = new char[strlen(m_szAlias) + strlen(m_szFirstName)
                       + strlen(m_szLastName) + strlen(m_szEmail)
                       + strlen(m_szReason) + strlen(m_szId) + strlen(p) + 256];
   //sprintf(m_szText, "%s (%s %s, %s), uin %lu, requests authorization to add you to their contact list:\n%s\n",
   //        m_szAlias, m_szFirstName, m_szLastName, m_szEmail, m_nUin, m_szReason);
-  int pos = sprintf(m_szText, "Alias: %s\nUser: %s (%s)\nName: %s %s\nEmail: %s\n",
+  int pos = sprintf(m_szText, tr("Alias: %s\nUser: %s (%s)\nName: %s %s\nEmail: %s\n"),
      m_szAlias, m_szId, p, m_szFirstName, m_szLastName, m_szEmail);
   delete [] p;
-#else
-  m_szText = new char[strlen(m_szAlias) + strlen(m_szFirstName)
-                      + strlen(m_szLastName) + strlen(m_szEmail)
-                      + strlen(m_szReason) + 256];
-  //sprintf(m_szText, "%s (%s %s, %s), uin %lu, requests authorization to add you to their contact list:\n%s\n",
-  //        m_szAlias, m_szFirstName, m_szLastName, m_szEmail, m_nUin, m_szReason);
-  int pos = sprintf(m_szText, "Alias: %s\nUin: %lu\nName: %s %s\nEmail: %s\n",
-     m_szAlias, m_nUin, m_szFirstName, m_szLastName, m_szEmail);
-#endif
+
   if (m_szReason[0] != '\0')
-    sprintf(&m_szText[pos], "Authorization Request:\n%s\n", m_szReason);
+    sprintf(&m_szText[pos], tr("Authorization Request:\n%s\n"), m_szReason);
 }
 
 
@@ -530,15 +505,12 @@ CEventAuthRequest::~CEventAuthRequest()
   free (m_szLastName);
   free (m_szEmail);
   free (m_szReason);
-#ifdef PROTOCOL_PLUGIN
   free (m_szId);
-#endif
 }
 
 
 void CEventAuthRequest::AddToHistory(ICQUser *u, direction _nDir)
 {
-#ifdef PROTOCOL_PLUGIN
   char *p = PPIDSTRING(m_nPPID);
   char *szOut = new char[(strlen(m_szAlias) + strlen(m_szFirstName) +
                     strlen(m_szLastName) + strlen(m_szEmail) +
@@ -548,24 +520,15 @@ void CEventAuthRequest::AddToHistory(ICQUser *u, direction _nDir)
   nPos += sprintf(&szOut[nPos], ":%s (%s)\n:%s\n:%s\n:%s\n:%s\n", m_szId,
                   p, m_szAlias, m_szFirstName, m_szLastName, m_szEmail);
   delete [] p;
-#else
-  char *szOut = new char[(strlen(m_szAlias) + strlen(m_szFirstName) +
-                    strlen(m_szLastName) + strlen(m_szEmail) +
-                    strlen(m_szReason)) * 2 + 16 +
-                   EVENT_HEADER_SIZE];
-  int nPos = AddToHistory_Header(_nDir, szOut);
-  nPos += sprintf(&szOut[nPos], ":%lu\n:%s\n:%s\n:%s\n:%s\n", m_nUin,
-                  m_szAlias, m_szFirstName, m_szLastName, m_szEmail);
-#endif
+
   AddStrWithColons(&szOut[nPos], m_szReason);
-  AddToHistory_Flush(u, szOut);
+  AddToHistory_Flush(u, szOut, m_nPPID);
   delete [] szOut;
 }
 
 
 
 //=====CEventAuthGranted========================================================
-#ifdef PROTOCOL_PLUGIN
 CEventAuthGranted::CEventAuthGranted(const char *_szId, unsigned long _nPPID,
                        const char *_szMessage, unsigned short _nCommand,
                        time_t _tTime, unsigned long _nFlags)
@@ -574,9 +537,11 @@ CEventAuthGranted::CEventAuthGranted(const char *_szId, unsigned long _nPPID,
   m_szMessage = _szMessage == NULL ? strdup("") : strdup(_szMessage);
   m_szId = strdup(_szId);
   m_nPPID = _nPPID;
-  m_nUin = 0;
+  if (_nPPID == LICQ_PPID)
+    m_nUin = strtoul(_szId, (char **)NULL, 10);
+  else
+    m_nUin = 0;
 }
-#endif
 
 CEventAuthGranted::CEventAuthGranted(unsigned long _nUin, const char *_szMessage,
                        unsigned short _nCommand, time_t _tTime,
@@ -584,27 +549,22 @@ CEventAuthGranted::CEventAuthGranted(unsigned long _nUin, const char *_szMessage
    : CUserEvent(ICQ_CMDxSUB_AUTHxGRANTED, _nCommand, 0, _tTime, _nFlags)
 {
   m_szMessage = _szMessage == NULL ? strdup("") : strdup(_szMessage);
-#ifdef PROTOCOL_PLUGIN
+
   char szUin[24];
   sprintf(szUin, "%lu", _nUin);
   m_szId = strdup(szUin);
   m_nPPID = LICQ_PPID;
-#endif
   m_nUin = _nUin;
 }
 
 void CEventAuthGranted::CreateDescription()
 {
   if (m_szText) delete [] m_szText;
-#ifdef PROTOCOL_PLUGIN
   char *p = PPIDSTRING(m_nPPID);
   m_szText = new char[strlen(m_szId) + strlen(p) + strlen(m_szMessage) + 128];
-  int pos = sprintf(m_szText, "User %s (%s) authorized you", m_szId, p);
+  int pos = sprintf(m_szText, tr("User %s (%s) authorized you"), m_szId, p);
   delete [] p;
-#else
-  m_szText = new char[strlen(m_szMessage) + 128];
-  int pos = sprintf(m_szText, "User %lu authorized you", m_nUin);
-#endif
+
   if (m_szMessage[0] != '\0')
     sprintf(&m_szText[pos], ":\n%s\n", m_szMessage);
   else
@@ -615,35 +575,26 @@ void CEventAuthGranted::CreateDescription()
 CEventAuthGranted::~CEventAuthGranted()
 {
   free (m_szMessage);
-#ifdef PROTOCOL_PLUGIN
   free (m_szId);
-#endif
 }
 
 void CEventAuthGranted::AddToHistory(ICQUser *u, direction _nDir)
 {
-#ifdef PROTOCOL_PLUGIN
   char *p = PPIDSTRING(m_nPPID);
   char *szOut = new char[(strlen(m_szId) + strlen(p) + strlen(m_szMessage))
                          * 2 + 16 + EVENT_HEADER_SIZE];
   int nPos = AddToHistory_Header(_nDir, szOut);
   nPos += sprintf(&szOut[nPos], ":%s (%s)\n", m_szId, p);
   delete [] p;
-#else
-  char *szOut = new char[strlen(m_szMessage) * 2 + 16 +
-                   EVENT_HEADER_SIZE];
-  int nPos = AddToHistory_Header(_nDir, szOut);
-  nPos += sprintf(&szOut[nPos], ":%lu\n", m_nUin);
-#endif
+
   AddStrWithColons(&szOut[nPos], m_szMessage);
-  AddToHistory_Flush(u, szOut);
+  AddToHistory_Flush(u, szOut, m_nPPID);
   delete [] szOut;
 }
 
 
 
 //=====CEventAuthRefused==========================================================
-#ifdef PROTOCOL_PLUGIN
 CEventAuthRefused::CEventAuthRefused(const char *_szId, unsigned long _nPPID,
                        const char *_szMessage, unsigned short _nCommand,
                        time_t _tTime, unsigned long _nFlags)
@@ -657,7 +608,6 @@ CEventAuthRefused::CEventAuthRefused(const char *_szId, unsigned long _nPPID,
   else
     m_nUin = 0;
 }
-#endif
 
 CEventAuthRefused::CEventAuthRefused(unsigned long _nUin, const char *_szMessage,
                        unsigned short _nCommand, time_t _tTime,
@@ -666,26 +616,21 @@ CEventAuthRefused::CEventAuthRefused(unsigned long _nUin, const char *_szMessage
 {
   m_szMessage = _szMessage == NULL ? strdup("") : strdup(_szMessage);
   m_nUin = _nUin;
-#ifdef PROTOCOL_PLUGIN
+  
   char szUin[24];
   sprintf(szUin, "%lu", _nUin);
   m_szId = strdup(szUin);
   m_nPPID = LICQ_PPID;
-#endif
 }
 
 void CEventAuthRefused::CreateDescription()
 {
   if (m_szText) delete [] m_szText;
-#ifdef PROTOCOL_PLUGIN
   char *p = PPIDSTRING(m_nPPID);
   m_szText = new char[strlen(m_szId) + strlen(p) + strlen(m_szMessage) + 128];
-  int pos = sprintf(m_szText, "User %s (%s) refused to authorize you", m_szId, p);
+  int pos = sprintf(m_szText, tr("User %s (%s) refused to authorize you"), m_szId, p);
   delete [] p;
-#else
-  m_szText = new char[strlen(m_szMessage) + 128];
-  int pos = sprintf(m_szText, "User %lu refused to authorize you", m_nUin);
-#endif
+
   if (m_szMessage[0] != '\0')
     sprintf(&m_szText[pos], ":\n%s\n", m_szMessage);
   else
@@ -696,28 +641,20 @@ void CEventAuthRefused::CreateDescription()
 CEventAuthRefused::~CEventAuthRefused()
 {
   free (m_szMessage);
-#ifdef PROTOCOL_PLUGIN
   free (m_szId);
-#endif
 }
 
 void CEventAuthRefused::AddToHistory(ICQUser *u, direction _nDir)
 {
-#ifdef PROTOCOL_PLUGIN
   char *p = PPIDSTRING(m_nPPID);
   char *szOut = new char[(strlen(m_szId) + strlen(p) + strlen(m_szMessage)) * 2 +
                          16 + EVENT_HEADER_SIZE];
   int nPos = AddToHistory_Header(_nDir, szOut);
   nPos += sprintf(&szOut[nPos], ":%s (%s)\n", m_szId, p);
   delete [] p;
-#else
-  char *szOut = new char[strlen(m_szMessage) * 2 + 16 +
-                   EVENT_HEADER_SIZE];
-  int nPos = AddToHistory_Header(_nDir, szOut);
-  nPos += sprintf(&szOut[nPos], ":%lu\n", m_nUin);
-#endif
+
   AddStrWithColons(&szOut[nPos], m_szMessage);
-  AddToHistory_Flush(u, szOut);
+  AddToHistory_Flush(u, szOut, m_nPPID);
   delete [] szOut;
 }
 
@@ -738,7 +675,7 @@ void CEventWebPanel::CreateDescription()
 {
   if (m_szText) delete [] m_szText;
   m_szText = new char[strlen(m_szName) + strlen(m_szEmail) + strlen(m_szMessage) + 64];
-  sprintf(m_szText, "Message from %s (%s) through web panel:\n%s\n",
+  sprintf(m_szText, tr("Message from %s (%s) through web panel:\n%s\n"),
           m_szName, m_szEmail, m_szMessage);
 }
 
@@ -779,7 +716,7 @@ void CEventEmailPager::CreateDescription()
 {
   if (m_szText) delete [] m_szText;
   m_szText = new char[strlen(m_szName) + strlen(m_szEmail) + strlen(m_szMessage) + 64];
-  sprintf(m_szText, "Message from %s (%s) through email pager:\n%s\n",
+  sprintf(m_szText, tr("Message from %s (%s) through email pager:\n%s\n"),
           m_szName, m_szEmail, m_szMessage);
 }
 
@@ -811,7 +748,7 @@ CEventContactList::CEventContactList(ContactList &cl, bool bDeep,
 {
   if (bDeep)
     for(ContactList::const_iterator it = cl.begin(); it != cl.end(); ++it)
-      m_vszFields.push_back(new CContact((*it)->Uin(), (*it)->Alias()));
+      m_vszFields.push_back(new CContact((*it)->IdString(), (*it)->PPID(), (*it)->Alias()));
   else
     m_vszFields = cl;
 }
@@ -822,15 +759,11 @@ void CEventContactList::CreateDescription()
   if (m_szText) delete [] m_szText;
   m_szText = new char [m_vszFields.size() * 32 + 128];
   char *szEnd = m_szText;
-  szEnd += sprintf(m_szText, "Contact list (%d contacts):\n", m_vszFields.size());
+  szEnd += sprintf(m_szText, tr("Contact list (%d contacts):\n"), m_vszFields.size());
   ContactList::const_iterator iter;
   for (iter = m_vszFields.begin(); iter != m_vszFields.end(); iter++)
   {
-#ifdef PROTOCOL_PLUGIN
     szEnd += sprintf(szEnd, "%s (%s)\n", (*iter)->Alias(), (*iter)->IdString());
-#else
-    szEnd += sprintf(szEnd, "%s (%lu)\n", (*iter)->Alias(), (*iter)->Uin());
-#endif
   }
 }
 
@@ -845,21 +778,19 @@ CEventContactList::~CEventContactList()
 
 void CEventContactList::AddToHistory(ICQUser *u, direction _nDir)
 {
+  unsigned long nPPID = LICQ_PPID;
   char *szOut = new char[m_vszFields.size() * 32 + EVENT_HEADER_SIZE];
   int nPos = AddToHistory_Header(_nDir, szOut);
   ContactList::const_iterator iter;
   for (iter = m_vszFields.begin(); iter != m_vszFields.end(); iter++)
   {
-#ifdef PROTOCOL_PLUGIN
     char *p = PPIDSTRING((*iter)->PPID());
+    nPPID = (*iter)->PPID();
     nPos += sprintf(&szOut[nPos], ":%s (%s)\n:%s\n", (*iter)->IdString(),
       p, (*iter)->Alias());
     delete [] p;
-#else
-    nPos += sprintf(&szOut[nPos], ":%lu\n:%s\n", (*iter)->Uin(), (*iter)->Alias());
-#endif
   }
-  AddToHistory_Flush(u, szOut);
+  AddToHistory_Flush(u, szOut, nPPID);
   delete [] szOut;
 }
 
@@ -882,7 +813,7 @@ CEventContactList *CEventContactList::Parse(char *sz, unsigned short nCmd, time_
   for (i = 0; i < nNumContacts * 2; i += 2)
   {
     gTranslator.ServerToClient(szFields[i + 1]);
-    vc.push_back(new CContact(atoi(szFields[i]), szFields[i + 1]));
+    vc.push_back(new CContact(szFields[i], LICQ_PPID, szFields[i + 1]));
   }
   delete[] szFields;
 
@@ -902,7 +833,7 @@ void CEventSms::CreateDescription()
 {
   if (m_szText) delete [] m_szText;
   m_szText = new char[strlen(m_szNumber) + strlen(m_szMessage) + 32];
-  sprintf(m_szText, "Phone: %s\n%s\n", m_szNumber, m_szMessage);
+  sprintf(m_szText, tr("Phone: %s\n%s\n"), m_szNumber, m_szMessage);
 }
 
 CEventSms::~CEventSms()
@@ -966,7 +897,7 @@ void CEventServerMessage::CreateDescription()
 {
   if (m_szText) delete [] m_szText;
   m_szText = new char[strlen(m_szName) + strlen(m_szEmail) + strlen(m_szMessage) + 64];
-  sprintf(m_szText, "System Server Message from %s (%s):\n%s\n", m_szName,
+  sprintf(m_szText, tr("System Server Message from %s (%s):\n%s\n"), m_szName,
           m_szEmail, m_szMessage);
 }
 
@@ -1070,33 +1001,33 @@ void CEventUnknownSysMsg::AddToHistory(ICQUser * /*u*/, direction /*_nDir*/)
 static const int MAX_EVENT = 26;
 
 static const char *szEventTypes[27] =
-{ "Plugin Event",
-  "Message",
-  "Chat Request",
-  "File Transfer",
-  "URL",
+{ tr("Plugin Event"),
+  tr("Message"),
+  tr("Chat Request"),
+  tr("File Transfer"),
+  tr("URL"),
   "",
-  "Authorization Request",
-  "AUthorization Refused",
-  "Authorization Granted",
-  "Server Message",
+  tr("Authorization Request"),
+  tr("AUthorization Refused"),
+  tr("Authorization Granted"),
+  tr("Server Message"),
   "",
   "",
-  "Added to Contact List",
-  "Web Panel",
-  "Email Pager",
+  tr("Added to Contact List"),
+  tr("Web Panel"),
+  tr("Email Pager"),
   "",
   "",
   "",
   "",
-  "Contact List",
+  tr("Contact List"),
   "",
   "",
   "",
   "",
   "",
   "",
-  "SMS"
+  tr("SMS")
 };
 
 
@@ -1107,12 +1038,12 @@ const char *CUserEvent::Description()
 
   if (SubCommand() > MAX_EVENT ||
       szEventTypes[SubCommand()][0] == '\0')
-    strcpy(desc, "Unknown Event");
+    strcpy(desc, tr("Unknown Event"));
   else
   {
     strcpy(desc, szEventTypes[SubCommand()]);
     if (IsCancelled())
-      strcat(desc, " (cancelled)");
+      strcat(desc, tr(" (cancelled)"));
   }
   return desc;
 }
