@@ -32,6 +32,9 @@ extern int errno;
 extern int h_errno;
 #endif
 
+// Localization
+#include "gettext.h"
+
 #include "licq_socket.h"
 #include "licq_icq.h"
 #include "licq_log.h"
@@ -44,6 +47,7 @@ extern int h_errno;
 #include <openssl/err.h>
 
 SSL_CTX *gSSL_CTX;
+SSL_CTX *gSSL_CTX_NONICQ;
 #endif // OpenSSL
 
 #ifdef USE_SOCKS5
@@ -167,7 +171,33 @@ char *INetSocket::RemoteIpStr(char *buf)
   return (inet_ntoa_r(*(struct in_addr *)&m_sRemoteAddr.sin_addr.s_addr, buf));
 }
 
+//-----INetSocket::SetOwner---------------------------------------------------
+void INetSocket::SetOwner(unsigned long _nOwner)
+{
+  m_nOwner = _nOwner;
+  if (_nOwner)
+  {
+    char szUin[24];
+    sprintf(szUin, "%lu", _nOwner);
+    m_szOwnerId = strdup(szUin);
+    m_nOwnerPPID = LICQ_PPID;
+  }
+  else
+  {
+    m_szOwnerId = 0;
+    m_nOwnerPPID = 0;
+  }
+}
 
+void INetSocket::SetOwner(const char *_szOwnerId, unsigned long _nOwnerPPID)
+{
+  m_szOwnerId = strdup(_szOwnerId);
+  m_nOwnerPPID = _nOwnerPPID;
+  if (m_nOwnerPPID == LICQ_PPID)
+    m_nOwner = strtoul(_szOwnerId, (char **)NULL, 10);
+  else
+    m_nOwner = 0;
+}
 
 //-----INetSocket::Error------------------------------------------------------
 int INetSocket::Error()
@@ -197,7 +227,7 @@ char *INetSocket::ErrorStr(char *buf, int buflen)
 
     case SOCK_ERROR_h_errno:
 #ifndef HAVE_HSTRERROR
-      sprintf(buf, "hostname resolution failure (%d)", h_errno);
+      sprintf(buf, tr("hostname resolution failure (%d)"), h_errno);
 #else
       strncpy(buf, hstrerror(h_errno), buflen);
       buf[buflen - 1] = '\0';
@@ -205,17 +235,17 @@ char *INetSocket::ErrorStr(char *buf, int buflen)
       break;
 
     case SOCK_ERROR_desx:
-      strncpy(buf, "DesX encryption/decryption failure", buflen);
+      strncpy(buf, tr("DesX encryption/decryption failure"), buflen);
       buf[buflen - 1] = '\0';
       break;
 
     case SOCK_ERROR_none:
-      strncpy(buf, "No error detected", buflen);
+      strncpy(buf, tr("No error detected"), buflen);
       buf[buflen - 1] = '\0';
       break;
 
     case SOCK_ERROR_internal:
-      strncpy(buf, "Internal error", buflen);
+      strncpy(buf, tr("Internal error"), buflen);
       buf[buflen - 1] = '\0';
       break;
     case SOCK_ERROR_proxy:
@@ -229,29 +259,19 @@ char *INetSocket::ErrorStr(char *buf, int buflen)
 
 
 //-----INetSocket::constructor--------------------------------------------------
-#ifdef PROTOCOL_PLUGIN
 INetSocket::INetSocket(unsigned long _nOwner)
 {
   m_nDescriptor = -1;
-  if (_nOwner)
-  {
-    char szUin[24];
-    sprintf(szUin, "%lu", _nOwner);
-    m_szOwnerId = strdup(szUin);
-    m_nOwnerPPID = LICQ_PPID;
-  }
-  else
-  {
-    m_szOwnerId = 0;
-    m_nOwnerPPID = 0;
-  }
+  SetOwner(_nOwner);
+
   m_nVersion = 0;
   m_nErrorType = SOCK_ERROR_none;
   memset(&m_sRemoteAddr, 0, sizeof(struct sockaddr_in));
   memset(&m_sLocalAddr, 0, sizeof(struct sockaddr_in));
   m_szRemoteName = NULL;
   m_xProxy = NULL;
-
+  m_nChannel = ICQ_CHNxNONE;
+  
   // Initialise the mutex
   pthread_mutex_init(&mutex, NULL);
 }
@@ -264,32 +284,21 @@ INetSocket::INetSocket(const char *_szOwnerId, unsigned long _nOwnerPPID)
   else
     m_szOwnerId = 0;
   m_nOwnerPPID = _nOwnerPPID;
+  if (m_nOwnerPPID == LICQ_PPID)
+    m_nOwner = strtoul(_szOwnerId, (char **)NULL, 10);
+  else
+    m_nOwner = 0;
   m_nVersion = 0;
   m_nErrorType = SOCK_ERROR_none;
   memset(&m_sRemoteAddr, 0, sizeof(struct sockaddr_in));
   memset(&m_sLocalAddr, 0, sizeof(struct sockaddr_in));
   m_szRemoteName = NULL;
   m_xProxy = NULL;
-
+  m_nChannel = ICQ_CHNxNONE;
+  
   // Initialize the mutex
   pthread_mutex_init(&mutex, NULL);
 }
-#else
-INetSocket::INetSocket(unsigned long _nOwner)
-{
-  m_nDescriptor = -1;
-  m_nOwner = _nOwner;
-  m_nVersion = 0;
-  m_nErrorType = SOCK_ERROR_none;
-  memset(&m_sRemoteAddr, 0, sizeof(struct sockaddr_in));
-  memset(&m_sLocalAddr, 0, sizeof(struct sockaddr_in));
-  m_szRemoteName = NULL;
-  m_xProxy = NULL;
-
-  // Initialise the mutex
-  pthread_mutex_init(&mutex, NULL);
-}
-#endif
 
 INetSocket::~INetSocket()
 {
@@ -303,10 +312,8 @@ INetSocket::~INetSocket()
     nResult = pthread_mutex_destroy(&mutex);
   } while (nResult != 0);
 
-#ifdef PROTOCOL_PLUGIN
   if (m_szOwnerId)
     free(m_szOwnerId);
-#endif
 
   if (m_szRemoteName != NULL) free (m_szRemoteName);
 }
@@ -387,8 +394,8 @@ bool INetSocket::SetLocalAddress(bool bIp)
   // This should never happen unless the IP stack is fucked
   if (m_sLocalAddr.sin_addr.s_addr == INADDR_ANY && bIp)
   {
-    gLog.Warn("%sYour IP stack or SOCKS client is a piece of crap.\n"
-              "%sAttempting to guess local IP.\n", L_WARNxSTR, L_BLANKxSTR);
+    gLog.Warn(tr("%sYour IP stack or SOCKS client is a piece of crap.\n"
+                 "%sAttempting to guess local IP.\n"), L_WARNxSTR, L_BLANKxSTR);
     char szHostName[256];
     if (gethostname(szHostName, 256) == -1)
     {
@@ -687,11 +694,11 @@ bool SrvSocket::RecvPacket()
     if (nBytesReceived <= 0)
     {
       if (nBytesReceived == 0)
-        gLog.Warn("server socket was closed!!!\n");
+        gLog.Warn(tr("server socket was closed!!!\n"));
       else {
         char buf[128];
         m_nErrorType = SOCK_ERROR_errno;
-        gLog.Warn("%serror during receiving from server socket :-((\n%s%s\n",
+        gLog.Warn(tr("%serror during receiving from server socket :-((\n%s%s\n"),
                   L_WARNxSTR, L_BLANKxSTR, ErrorStr(buf, sizeof(buf)));
       }
       delete[] buffer;
@@ -770,6 +777,7 @@ void TCPSocket::RecvConnection(TCPSocket &newSocket)
   newSocket.SetLocalAddress();
 }
 
+#define m_pSSL ((SSL *) m_p_SSL)
 
 /*-----TCPSocket::TransferConnectionFrom---------------------------------------
  * Transfers a connection from the given socket to the current one and closes
@@ -780,15 +788,18 @@ void TCPSocket::TransferConnectionFrom(TCPSocket &from)
   m_nDescriptor = from.m_nDescriptor;
   m_sLocalAddr = from.m_sLocalAddr;
   m_sRemoteAddr = from.m_sRemoteAddr;
-#ifdef PROTOCOL_PLUGIN
+
   if (m_szOwnerId)
     free (m_szOwnerId);
   if (from.m_szOwnerId)
     m_szOwnerId = strdup(from.m_szOwnerId);
+  else
+    m_szOwnerId = 0;
   m_nOwnerPPID = from.m_nOwnerPPID;
-#else
-  m_nOwner = from.m_nOwner;
-#endif
+
+  if (m_nOwnerPPID == LICQ_PPID)
+    m_nOwner = from.m_nOwner;
+
   m_nVersion = from.m_nVersion;
   if (from.m_p_SSL)
   {
@@ -808,8 +819,6 @@ void TCPSocket::TransferConnectionFrom(TCPSocket &from)
   from.CloseConnection();
 }
 
-
-
 /*-----TCPSocket::SendPacket---------------------------------------------------
  * Sends a packet on a socket.  The socket is blocking, so we are guaranteed
  * that the entire packet will be sent, however, it may block if the tcp
@@ -1056,6 +1065,93 @@ bool TCPSocket::RecvPacket()
 }
 
 
+bool TCPSocket::SSLSend(CBuffer *b_in)
+{
+#ifdef USE_OPENSSL
+  if (m_pSSL == 0) return false;
+
+  int i, j;
+  ERR_clear_error();
+  pthread_mutex_lock(&mutex_ssl);
+  i = SSL_write(m_pSSL, b_in->getDataStart(), b_in->getDataSize());
+  j = SSL_get_error(m_pSSL, i);
+  pthread_mutex_unlock(&mutex_ssl);
+  if (j != SSL_ERROR_NONE)
+  {
+    const char *file; int line;
+    unsigned long err;
+    switch (j)
+    {
+      case SSL_ERROR_SSL:
+        err = ERR_get_error_line(&file, &line);
+        printf("SSL_write error = %lx, %s:%i\n", err, file, line);
+        ERR_clear_error();
+        break;
+      default:
+        printf("SSL_write error %d, SSL_%d\n", i, j);
+        break;
+    }
+  }
+ 
+  DumpPacket(b_in, D_SENDER);
+
+  return true;
+#else
+  return false;
+#endif
+}
+
+
+bool TCPSocket::SSLRecv()
+{
+#ifdef USE_OPENSSL
+  if (m_pSSL == 0) return false;
+
+  char *buffer = new char[MAX_RECV_SIZE];
+  errno = 0;
+  pthread_mutex_lock(&mutex_ssl);
+  int nBytesReceived = SSL_read(m_pSSL, buffer, MAX_RECV_SIZE);
+  int tmp = SSL_get_error(m_pSSL, nBytesReceived);
+  pthread_mutex_unlock(&mutex_ssl);
+  switch (tmp)
+  {
+    case SSL_ERROR_NONE:
+      break;
+    case SSL_ERROR_WANT_READ:
+    case SSL_ERROR_WANT_WRITE:
+    case SSL_ERROR_WANT_X509_LOOKUP:
+      return (true);
+    case SSL_ERROR_ZERO_RETURN:
+      m_nErrorType = SOCK_ERROR_errno;
+      errno = 0;
+      return (false);
+    case SSL_ERROR_SYSCALL:
+      m_nErrorType = SOCK_ERROR_errno;
+      return (false);
+    case SSL_ERROR_SSL:
+      m_nErrorType = SOCK_ERROR_internal;
+      return (false);
+  }
+  if (nBytesReceived <= 0)
+  {
+    delete[] buffer;
+    m_nErrorType = SOCK_ERROR_errno;
+    return (false);
+  }
+  m_xRecvBuffer.Create(nBytesReceived);
+  m_xRecvBuffer.Pack(buffer, nBytesReceived);
+  delete[] buffer;
+
+  // Print the packet
+  DumpPacket(&m_xRecvBuffer, D_RECEIVER);
+
+  return (true);
+#else
+  return false;
+#endif
+}
+
+
 TCPSocket::~TCPSocket()
 {
   SecureStop();
@@ -1078,7 +1174,10 @@ bool TCPSocket::SSL_Pending()
 bool TCPSocket::SecureConnect()
 {
   pthread_mutex_init(&mutex_ssl, NULL);
-  m_pSSL = SSL_new(gSSL_CTX);
+  if (m_nOwnerPPID == LICQ_PPID)
+    m_p_SSL = SSL_new(gSSL_CTX);
+  else
+    m_p_SSL = SSL_new(gSSL_CTX_NONICQ);
 #ifdef SSL_DEBUG
   m_pSSL->debug = 1;
 #endif
@@ -1111,7 +1210,10 @@ bool TCPSocket::SecureListen()
 {
   pthread_mutex_init(&mutex_ssl, NULL);
 
-  m_pSSL = SSL_new(gSSL_CTX);
+  if (m_nOwnerPPID == LICQ_PPID)
+    m_p_SSL = SSL_new(gSSL_CTX);
+  else
+    m_p_SSL = SSL_new(gSSL_CTX_NONICQ);
   SSL_set_session(m_pSSL, NULL);
   SSL_set_fd(m_pSSL, m_nDescriptor);
   int i = SSL_accept(m_pSSL);
@@ -1341,13 +1443,10 @@ void CSocketManager::CloseSocket (int nSd, bool bClearUser, bool bDelete)
     return;
   }
 
-#ifdef PROTOCOL_PLUGIN
   char *szOwner = s->OwnerId() ? strdup(s->OwnerId()) : 0;
   unsigned long nPPID = s->OwnerPPID();
-#else
-  unsigned long nOwner = s->Owner();
-#endif
-
+  unsigned char nChannel = s->Channel();
+  
   // First remove the socket from the hash table so it won't be fetched anymore
   m_hSockets.Remove(nSd);
 
@@ -1362,19 +1461,16 @@ void CSocketManager::CloseSocket (int nSd, bool bClearUser, bool bDelete)
 
   if (bClearUser)
   {
-#ifdef PROTOCOL_PLUGIN
     ICQUser *u = NULL;
     if (szOwner)
     {
       u = gUserManager.FetchUser(szOwner, nPPID, LOCK_W);
       free(szOwner);
     }
-#else
-    ICQUser *u = gUserManager.FetchUser(nOwner, LOCK_W);
-#endif
+    
     if (u != NULL)
     {
-      u->ClearSocketDesc();
+      u->ClearSocketDesc(nChannel);
       if (u->OfflineOnDisconnect())
         gLicqDaemon->ChangeUserStatus(u, ICQ_STATUS_OFFLINE);
       gUserManager.DropUser(u);
